#!/usr/bin/env python
#
# Copyright 2019 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import time
from gnuradio import gr, gr_unittest, iio, blocks

import pmt


class test_iio(gr_unittest.TestCase):

    def test_import(self):
        """Just see if we can import the module...
        They may not have a IIO device connected, etc.  Don't try to run anything"""
        pass

    def test_attribute_updater(self):
        attr = "Test"
        val = "100"
        key0 = pmt.intern(attr)
        val0 = pmt.intern(val)
        msg_dic = pmt.make_dict()
        msg_dic = pmt.dict_add(msg_dic, key0, val0)

        src = iio.attr_updater(attr, val, 500)
        snk = blocks.message_debug()

        tb = gr.top_block()
        tb.msg_connect((src, "out"), (snk, "store"))
        tb.start()
        time.sleep(1)
        tb.stop()
        tb.wait()

        rec_msg = snk.get_message(0)
        self.assertTrue(pmt.equal(rec_msg, msg_dic))


if __name__ == '__main__':
    gr_unittest.run(test_iio)



#!/usr/bin/env python
#
# Copyright 2006,2007,2010,2015 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest
import numpy as np


class test_random(gr_unittest.TestCase):
    # NOTE: For tests on the output distribution of the random numbers, see
    # gnuradio-runtime/apps/evaluation_random_numbers.py.

    # Check for range [0,1) of uniform distributed random numbers
    def test_1(self):
        num_tests = 10000
        values = np.zeros(num_tests)
        rndm = gr.random()
        for k in range(num_tests):
            values[k] = rndm.ran1()
        for value in values:
            self.assertLess(value, 1)
            self.assertGreaterEqual(value, 0)

    # Same seed should yield same random values.
    def test_2_same_seed(self):
        num = 5
        # Init with fixed seed.
        rndm0 = gr.random(42)
        rndm1 = gr.random(42)
        for k in range(num):
            x = rndm0.ran1()
            y = rndm1.ran1()
            self.assertEqual(x, y)

    # reseed should yield same numbers.
    def test_003_reseed(self):
        num = 5
        x = np.zeros(num)
        y = np.zeros(num)
        rndm = gr.random(43)  # init with fix seed 1
        for k in range(num):
            x[k] = rndm.ran1()
        rndm.reseed(43)  # init with fix seed 2
        for k in range(num):
            y[k] = rndm.ran1()
        self.assertFloatTuplesAlmostEqual(x, y)

    def test_004_integer(self):
        nitems = 100000
        minimum = 2
        maximum = 42
        rng = gr.random(1, minimum, maximum)
        rnd_vals = np.zeros(nitems, dtype=int)
        for i in range(nitems):
            rnd_vals[i] = rng.ran_int()
        self.assertGreaterEqual(minimum, np.min(rnd_vals))
        self.assertLess(np.max(rnd_vals), maximum)

    def test_005_xoroshiro128p_seed_stability(self):
        """
        Test that seeding is stable.
        It's basically an API break if it isn't.

        We simply check for the first value of a sequence
        being the same as it was when the module was integrated.
        """
        rng = gr.xoroshiro128p_prng(42)
        self.assertEqual(3520422898491873512, rng())

    def test_006_xoroshiro128p_reproducibility(self):
        """
        Make sure two RNGs with the same seed yield the same
        sequence
        """
        seed = 123456
        N = 10000
        rng1 = gr.xoroshiro128p_prng(123456)
        rng2 = gr.xoroshiro128p_prng(123456)
        self.assertSequenceEqual(
            tuple(rng1() for _ in range(N)),
            tuple(rng2() for _ in range(N)))

    def test_007_xoroshiro128p_range(self):
        """
        Check bounds.
        Check whether a long sequence of values are within that bounds.
        """
        N = 10**6

        self.assertEqual(gr.xoroshiro128p_prng.min(), 0)
        self.assertEqual(gr.xoroshiro128p_prng.max(), 2**64 - 1)
        rng = gr.xoroshiro128p_prng(42)
        arr = all((0 <= rng() <= 2**64 - 1 for _ in range(N)))
        self.assertTrue(arr)


if __name__ == '__main__':
    gr_unittest.run(test_random)



import operator

import pytest

from grc.core.utils import expr_utils

id_getter = operator.itemgetter(0)
expr_getter = operator.itemgetter(1)


def test_simple():
    objects = [
        ['c', '2 * a + b'],
        ['a', '1'],
        ['b', '2 * a + unknown * d'],
        ['d', '5'],
    ]

    expected = [
        ['d', '5'],
        ['a', '1'],
        ['b', '2 * a + unknown * d'],
        ['c', '2 * a + b'],
    ]

    out = expr_utils.sort_objects(objects, id_getter, expr_getter)

    assert out == expected


def test_circular():
    test = [
        ['c', '2 * a + b'],
        ['a', '1'],
        ['b', '2 * c + unknown'],
    ]

    # Should fail due to circular dependency
    with pytest.raises(Exception):
        expr_utils.sort_objects(test, id_getter, expr_getter)



#
# Copyright 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" Automatically create YAML bindings for GRC from block code """


import os
import re
import glob
import logging
import yaml

from collections import OrderedDict

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except:
    from yaml import Loader, Dumper

try:
    from gnuradio.blocktool.core import Constants
except ImportError:
    have_blocktool = False
else:
    have_blocktool = True

from ..tools import ParserCCBlock, CMakeFileEditor, ask_yes_no, GRCYAMLGenerator
from .base import ModTool, ModToolException


logger = logging.getLogger(__name__)

## setup dumper for dumping OrderedDict ##
_MAPPING_TAG = yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG


def dict_representer(dumper, data):
    """ Representer to represent special OrderedDict """
    return dumper.represent_dict(data.items())


def dict_constructor(loader, node):
    """ Construct an OrderedDict for dumping """
    return OrderedDict(loader.construct_pairs(node))


Dumper.add_representer(OrderedDict, dict_representer)
Loader.add_constructor(_MAPPING_TAG, dict_constructor)


class ModToolMakeYAML(ModTool):
    """ Make YAML file for GRC block bindings """
    name = 'makeyaml'
    description = 'Generate YAML files for GRC block bindings.'

    def __init__(self, blockname=None, **kwargs):
        ModTool.__init__(self, blockname, **kwargs)
        self.info['pattern'] = blockname

    def validate(self):
        """ Validates the arguments """
        ModTool._validate(self)
        if not self.info['pattern'] or self.info['pattern'].isspace():
            raise ModToolException("Incorrect blockname (Regex)!")

    def run(self):
        """ Go, go, go! """
        # This portion will be covered by the CLI
        if not self.cli:
            self.validate()
        logger.warning(
            "Warning: This is an experimental feature. Don't expect any magic.")
        # 1) Go through lib/
        if not self.skip_subdirs['lib']:
            if self.info['version'] in ('37', '38', '310'):
                files = self._search_files('lib', '*_impl.cc')
            else:
                files = self._search_files('lib', '*.cc')
            for f in files:
                if os.path.basename(f)[0:2] == 'qa':
                    continue
                (params, iosig, blockname) = self._parse_cc_h(f)
                self._make_grc_yaml_from_block_data(params, iosig, blockname)
        # 2) Go through python/
        # TODO

    def _search_files(self, path, path_glob):
        """ Search for files matching pattern in the given path. """
        files = sorted(glob.glob(f"{path}/{path_glob}"))
        files_filt = []
        logger.info(f"Searching for matching files in {path}/:")
        if self.info['blockname']:
            # ensure the blockname given is not confused with similarly named blocks
            blockname_pattern = ''
            if path == 'python':
                blockname_pattern = f"^(qa_)?{self.info['blockname']}.py$"
            elif path == 'python/bindings':
                blockname_pattern = f"^{self.info['blockname']}_python.cc$"
            elif path == 'python/bindings/docstrings':
                blockname_pattern = f"^{self.info['blockname']}_pydoc_template.h$"
            elif path == 'lib':
                blockname_pattern = f"^{self.info['blockname']}_impl(\\.h|\\.cc)$"
            elif path == self.info['includedir']:
                blockname_pattern = f"^{self.info['blockname']}.h$"
            elif path == 'grc':
                blockname_pattern = f"^{self.info['modname']}_{self.info['blockname']}.block.yml$"
            for f in files:
                if re.search(blockname_pattern, os.path.basename(f)) is not None:
                    files_filt.append(f)
        elif self.info['pattern']:
            # search for block names matching a given regex pattern
            for f in files:
                if re.search(self.info['pattern'], os.path.basename(f)) is not None:
                    files_filt.append(f)
        if len(files_filt) == 0:
            logger.info("None found.")
        return files_filt

    def _make_grc_yaml_from_block_data(self, params, iosig, blockname):
        """ Take the return values from the parser and call the YAML
        generator. Also, check the makefile if the .yml file is in there.
        If necessary, add. """
        fname_yml = f'{self.info["modname"]}_{blockname}.block.yml'
        path_to_yml = os.path.join('grc', fname_yml)
        # Some adaptions for the GRC
        for inout in ('in', 'out'):
            if iosig[inout]['max_ports'] == '-1':
                iosig[inout]['max_ports'] = f'$num_{inout}puts'
                params.append({'key': f'num_{inout}puts',
                               'type': 'int',
                               'name': f'Num {inout}puts',
                               'default': '2',
                               'in_constructor': False})
        file_exists = False
        if os.path.isfile(path_to_yml):
            if not self.info['yes']:
                if not ask_yes_no('Overwrite existing GRC file?', False):
                    return
            else:
                file_exists = True
                logger.warning("Warning: Overwriting existing GRC file.")
        grc_generator = GRCYAMLGenerator(
            modname=self.info['modname'],
            blockname=blockname,
            params=params,
            iosig=iosig
        )
        grc_generator.save(path_to_yml)
        if file_exists:
            self.scm.mark_files_updated((path_to_yml,))
        else:
            self.scm.add_files((path_to_yml,))
        if not self.skip_subdirs['grc']:
            ed = CMakeFileEditor(self._file['cmgrc'])
            if re.search(fname_yml, ed.cfile) is None and not ed.check_for_glob('*.yml'):
                logger.info("Adding GRC bindings to grc/CMakeLists.txt...")
                ed.append_value('install', fname_yml,
                                to_ignore_end='DESTINATION[^()]+')
                ed.write()
                self.scm.mark_files_updated(self._file['cmgrc'])

    def _parse_cc_h(self, fname_cc):
        """ Go through a .cc and .h-file defining a block and return info """
        def _type_translate(p_type, default_v=None):
            """ Translates a type from C++ to GRC """
            translate_dict = {'float': 'float',
                              'double': 'real',
                              'int': 'int',
                              'gr_complex': 'complex',
                              'char': 'byte',
                              'unsigned char': 'byte',
                              'std::string': 'string',
                              'std::vector<int>': 'int_vector',
                              'std::vector<float>': 'real_vector',
                              'std::vector<gr_complex>': 'complex_vector',
                              }
            if p_type in ('int',) and default_v is not None and len(default_v) > 1 and default_v[:2].lower() == '0x':
                return 'hex'
            try:
                return translate_dict[p_type]
            except KeyError:
                return 'raw'

        def _get_blockdata(fname_cc):
            """ Return the block name and the header file name from the .cc file name """
            blockname = os.path.splitext(os.path.basename(
                fname_cc.replace('_impl.', '.')))[0]
            fname_h = (blockname + '.h').replace('_impl.', '.')
            contains_modulename = blockname.startswith(
                self.info['modname'] + '_')
            blockname = blockname.replace(self.info['modname'] + '_', '', 1)
            return (blockname, fname_h, contains_modulename)
        # Go, go, go
        logger.info(f"Making GRC bindings for {fname_cc}...")
        (blockname, fname_h, contains_modulename) = _get_blockdata(fname_cc)
        try:
            parser = ParserCCBlock(fname_cc,
                                   os.path.join(
                                       self.info['includedir'], fname_h),
                                   blockname,
                                   self.info['version'],
                                   _type_translate
                                   )
        except IOError:
            raise ModToolException(
                f"Can't open some of the files necessary to parse {fname_cc}.")

        if contains_modulename:
            return (parser.read_params(), parser.read_io_signature(), self.info['modname'] + '_' + blockname)
        else:
            return (parser.read_params(), parser.read_io_signature(), blockname)


def yaml_generator(self, **kwargs):
    """
    Generate YAML file from the block header file using blocktool API
    """
    header = self.filename.split('.')[0]
    block = self.modname.split('-')[-1]
    label = header.split('_')
    del label[-1]
    yml_file = os.path.join('.', block + '_' + header + '.block.yml')
    _header = (('id', f'{block}_{ header}'),
               ('label', ' '.join(label).upper()),
               ('category', f'[{block.capitalize()}]'),
               ('flags', '[python, cpp]')
               )
    params_list = [
        '${' + s['name'] + '}' for s in self.parsed_data['properties'] if self.parsed_data['properties']]
    str_ = ', '.join(params_list)
    _templates = [('imports', f'from gnuradio import {block}'),
                  ('make', f'{block}.{ header}({str_})')
                  ]

    if self.parsed_data['methods']:
        list_callbacks = []
        for param in self.parsed_data['methods']:
            arguments = []
            for args in param['arguments_type']:
                arguments.append(args['name'])
            arg_list = ['${' + s + '}' for s in arguments if arguments]
            arg_ = ', '.join(arg_list)
            list_callbacks.append(
                param['name'] + f'({arg_})')
        callback_key = ('callbacks')
        callbacks = (callback_key, tuple(list_callbacks))
        _templates.append(callbacks)
    _templates = tuple(_templates)

    data = OrderedDict()
    for tag, value in _header:
        data[tag] = value

    templates = OrderedDict()
    for tag, value in _templates:
        templates[tag] = value
    data['templates'] = templates

    parameters = []
    for param in self.parsed_data['properties']:
        parameter = OrderedDict()
        parameter['id'] = param['name']
        parameter['label'] = param['name'].capitalize()
        parameter['dtype'] = param['dtype']
        parameter['read_only'] = param['read_only']
        parameters.append(parameter)
    if parameters:
        data['parameters'] = parameters

    input_signature = []
    max_input_port = self.parsed_data['io_signature']['input']['max_streams']
    i_sig = self.parsed_data['io_signature']['input']['signature']
    for port in range(0, int(max_input_port)):
        input_sig = OrderedDict()
        if i_sig is Constants.MAKE:
            input_sig['domain'] = 'stream'
            input_sig['dtype'] = self.parsed_data['io_signature']['input']['sizeof_stream_item']
        elif i_sig is Constants.MAKE2:
            input_sig['domain'] = 'stream'
            input_sig['dtype'] = self.parsed_data['io_signature']['input']['sizeof_stream_item' +
                                                                           str(port + 1)]
        elif i_sig is Constants.MAKE3:
            input_sig['domain'] = 'stream'
            input_sig['dtype'] = self.parsed_data['io_signature']['input']['sizeof_stream_item' +
                                                                           str(port + 1)]
        elif i_sig is Constants.MAKEV:
            input_sig['domain'] = 'stream'
            input_sig['dtype'] = self.parsed_data['io_signature']['input']['sizeof_stream_items']
        input_signature.append(input_sig)

    if self.parsed_data['message_port']['input']:
        for _input in self.parsed_data['message_port']['input']:
            m_input_sig = OrderedDict()
            m_input_sig['domain'] = 'message'
            m_input_sig['id'] = _input
            input_signature.append(m_input_sig)
    if input_signature:
        data['inputs'] = input_signature

    output_signature = []
    max_output_port = self.parsed_data['io_signature']['output']['max_streams']
    o_sig = self.parsed_data['io_signature']['output']['signature']
    for port in range(0, int(max_output_port)):
        output_sig = OrderedDict()
        if o_sig is Constants.MAKE:
            output_sig['domain'] = 'stream'
            output_sig['dtype'] = self.parsed_data['io_signature']['output']['sizeof_stream_item']
        elif o_sig is Constants.MAKE2:
            output_sig['domain'] = 'stream'
            output_sig['dtype'] = self.parsed_data['io_signature']['output']['sizeof_stream_item' +
                                                                             str(port + 1)]
        elif o_sig is Constants.MAKE3:
            output_sig['domain'] = 'stream'
            output_sig['dtype'] = self.parsed_data['io_signature']['output']['sizeof_stream_item' +
                                                                             str(port + 1)]
        elif o_sig is Constants.MAKEV:
            output_sig['domain'] = 'stream'
            output_sig['dtype'] = self.parsed_data['io_signature']['output']['sizeof_stream_items']
        output_signature.append(output_sig)

    if self.parsed_data['message_port']['output']:
        for _output in self.parsed_data['message_port']['output']:
            m_output_sig = OrderedDict()
            m_output_sig['domain'] = 'message'
            m_output_sig['id'] = _output
            output_signature.append(m_output_sig)
    if output_signature:
        data['outputs'] = output_signature

    param_ = ', '.join(params_list)
    _cpp_templates = [('includes', '#include <gnuradio/{block}/{self.filename}>'),
                      ('declarations', '{block}::{ header}::sptr ${{id}}'),
                      ('make',
                       'this->${{id}} = {block}::{ header}::make({param_})')
                      ]

    if self.parsed_data['methods']:
        list_callbacks = []
        for param in self.parsed_data['methods']:
            arguments = []
            for args in param['arguments_type']:
                arguments.append(args['name'])
            arg_list = ['${' + s + '}' for s in arguments if arguments]
            arg_ = ', '.join(arg_list)
            list_callbacks.append(
                param['name'] + f'({arg_})')
        callback_key = ('callbacks')
        callbacks = (callback_key, tuple(list_callbacks))
        _cpp_templates.append(callbacks)

    link = ('link', 'gnuradio-{block}')
    _cpp_templates.append(link)
    _cpp_templates = tuple(_cpp_templates)

    cpp_templates = OrderedDict()
    for tag, value in _cpp_templates:
        cpp_templates[tag] = value
    data['cpp_templates'] = cpp_templates

    if self.parsed_data['docstring'] is not None:
        data['documentation'] = self.parsed_data['docstring']
    data['file_format'] = 1

    if kwargs['output']:
        with open(yml_file, 'w') as yml:
            yaml.dump(data, yml, Dumper=Dumper, default_flow_style=False)
    else:
        print(yaml.dump(data, Dumper=Dumper, allow_unicode=True,
                        default_flow_style=False, indent=4))



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import cmath

from gnuradio import gr, gr_unittest, analog, blocks


class test_quadrature_demod(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_quad_demod_001(self):
        f = 1000.0
        fs = 8000.0

        src_data = []
        for i in range(200):
            ti = i / fs
            src_data.append(cmath.exp(2j * cmath.pi * f * ti))

        # f/fs is a quarter turn per sample.
        # Set the gain based on this to get 1 out.
        gain = 1.0 / (cmath.pi / 4)

        expected_result = [0, ] + 199 * [1.0]

        src = blocks.vector_source_c(src_data)
        op = analog.quadrature_demod_cf(gain)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)

    def test_fm_end_to_end_002(self):
        f = 1000.0
        fs = 8000.0
        src = analog.sig_source_f(fs, analog.gr_waveform_t.GR_COS_WAVE, f, 1.0)
        head = blocks.head(gr.sizeof_float, 100)
        sensitivity = cmath.pi / 4
        fm = analog.frequency_modulator_fc(sensitivity)
        quad_demod = analog.quadrature_demod_cf(1.0 / sensitivity)
        sink_signal = blocks.vector_sink_f()
        sink_demod = blocks.vector_sink_f()
        self.tb.connect(src, head, sink_signal)
        self.tb.connect(head, fm, quad_demod, sink_demod)
        self.tb.run()
        expected_result = sink_signal.data()
        expected_result[0] = 0.0
        result_data = sink_demod.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 5)


if __name__ == '__main__':
    gr_unittest.run(test_quadrature_demod)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import sys
from PyQt5 import QtWidgets
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure

from gnuradio import gr
import pmt


class DistanceRadar(gr.sync_block, FigureCanvas):
    """
    This block creates a radar-like screen used to represent distance or size.
    This can be used in many ways such as circles closer to the center are
    closer, or just the opposite where closer to the center is smaller.

    Note: Incoming values should range between 0 (center bullseye) and
    100 (all the way out)
    """

    def __init__(self, lbl, ticklabels, backgroundColor, fontColor, ringColor, Parent=None,
                 width=4, height=4, dpi=100):
        gr.sync_block.__init__(self, name="distanceradar",
                               in_sig=None, out_sig=None)

        self.lbl = lbl

        self.message_port_register_in(pmt.intern("radius"))
        self.set_msg_handler(pmt.intern("radius"), self.msgHandler)

        self.fontColor = fontColor
        self.backgroundColor = backgroundColor
        self.ringColor = ringColor

        self.fig = Figure(figsize=(width, height), dpi=dpi)
        self.fig.patch.set_facecolor(self.backgroundColor)
        self.axes = self.fig.add_subplot(
            111, polar=True, facecolor=self.backgroundColor)

        # Create an "invisible" line at 100 to set the max for the plot
        self.axes.plot(np.linspace(0, 2 * np.pi, 100), np.ones(100) * 100, color=self.fontColor,
                       linestyle='')

        # Plot line: Initialize out to 100 and blank
        radius = 100
        self.blackline = self.axes.plot(np.linspace(0, 2 * np.pi, 100), np.ones(100) * radius,
                                        color=self.fontColor, linestyle='-')
        self.redline = None

        self.filledcircle = None
        # Create bullseye
        circle = plt.Circle((0.0, 0.0), 20, transform=self.axes.transData._b, color=self.fontColor,
                            alpha=0.4)
        self.bullseye = self.axes.add_artist(circle)

        # Rotate zero up
        self.axes.set_theta_zero_location("N")

        self.axes.set_yticklabels(ticklabels, color=self.fontColor)
        self.axes.set_xticklabels([], color=self.fontColor)

        FigureCanvas.__init__(self, self.fig)
        self.setParent(Parent)

        self.title = self.fig.suptitle(self.lbl, fontsize=8, fontweight='bold',
                                       color=self.fontColor)

        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)
        self.setMinimumSize(240, 230)
        FigureCanvas.updateGeometry(self)

    def msgHandler(self, msg):
        try:
            new_val = pmt.to_python(pmt.cdr(msg))

            if type(new_val) == float or type(new_val) == int:
                self.updateData(new_val)
            else:
                gr.log.error("Value received was not an int or a "
                             "float: %s" % str(type(new_val)))

        except Exception as e:
            gr.log.error("Error with message conversion: %s" % str(e))

    def updateData(self, radius):
        if self.redline is not None:
            self.redline.pop(0).remove()
        self.redline = self.axes.plot(np.linspace(0, 2 * np.pi, 100), np.ones(100) * radius,
                                      color='r', linestyle='-')

        if self.filledcircle:
            self.filledcircle.remove()

        self.bullseye.remove()
        circle = plt.Circle((0.0, 0.0), radius, transform=self.axes.transData._b,
                            color=self.ringColor, alpha=0.4)
        self.filledcircle = self.axes.add_artist(circle)
        # Create bullseye
        circle = plt.Circle((0.0, 0.0), 20, transform=self.axes.transData._b,
                            color=self.fontColor, alpha=0.4)
        self.bullseye = self.axes.add_artist(circle)

        self.draw()



#!/usr/bin/env python
#
# Copyright 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest, blocks
import os
import tempfile
import pmt


class test_file_descriptor_source_sink(gr_unittest.TestCase):

    def setUp(self):
        os.environ['GR_CONF_CONTROLPORT_ON'] = 'False'
        self.tb = gr.top_block()
        temp = tempfile.NamedTemporaryFile(delete=False)
        temp.close()
        self._datafilename = temp.name

    def tearDown(self):
        self.tb = None
        os.unlink(self._datafilename)

    def test_file_descriptor(self):
        src_data = range(1000)
        expected_result = range(1000)

        snk2 = blocks.vector_sink_f()

        binary = os.O_BINARY if os.name == "nt" else 0
        fd0 = os.open(self._datafilename, os.O_WRONLY | binary)

        src = blocks.vector_source_f(src_data)
        snk = blocks.file_descriptor_sink(gr.sizeof_float, fd0)

        self.tb.connect(src, snk)
        self.tb.run()

        fd1 = os.open(self._datafilename, os.O_RDONLY | binary)
        src2 = blocks.file_descriptor_source(gr.sizeof_float, fd1, False)

        self.tb.disconnect(src, snk)
        self.tb.connect(src2, snk2)
        self.tb.run()

        result_data = snk2.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)
        self.assertEqual(len(snk2.tags()), 0)


if __name__ == '__main__':
    gr_unittest.run(test_file_descriptor_source_sink)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2004,2010,2013,2016 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import time

from gnuradio import gr, gr_unittest, blocks
import pmt


# def all_counts():
#     return (gr.block_ncurrently_allocated(),
#             gr.block_detail_ncurrently_allocated(),
#             gr.buffer_ncurrently_allocated(),
#             gr.buffer_reader_ncurrently_allocated(),
#             gr.message_ncurrently_allocated())


class test_message(gr_unittest.TestCase):

    # def setUp(self):
    #     # self.msgq = gr.msg_queue()

    # def tearDown(self):
    #     # self.msgq = None

    # def leak_check(self, fct):
    #     begin = all_counts()
    #     fct()
    #     # tear down early so we can check for leaks
    #     self.tearDown()
    #     end = all_counts()
    #     self.assertEqual(begin, end)

    def test_100(self):
        msg = gr.message(0, 1.5, 2.3)
        self.assertEqual(0, msg.type())
        self.assertAlmostEqual(1.5, msg.arg1())
        self.assertAlmostEqual(2.3, msg.arg2())
        self.assertEqual(0, msg.length())

    def test_101(self):
        s = b'This is a test'
        msg = gr.message_from_string(s.decode('utf8'))
        self.assertEqual(s, msg.to_string())

    def test_102_unicodechars(self):
        s = u"(╯°□°)╯︵ ┻━┻"
        msg = gr.message_from_string(s)
        self.assertEqual(s.encode('utf8'), msg.to_string())

    # msg_queue was removed from API in 3.8
    # def test_200(self):
    #     self.leak_check(self.body_200)

    # def body_200(self):
    #     self.msgq.insert_tail(gr.message(0))
    #     self.assertEqual(1, self.msgq.count())
    #     self.msgq.insert_tail(gr.message(1))
    #     self.assertEqual(2, self.msgq.count())
    #     msg0 = self.msgq.delete_head()
    #     self.assertEqual(0, msg0.type())
    #     msg1 = self.msgq.delete_head()
    #     self.assertEqual(1, msg1.type())
    #     self.assertEqual(0, self.msgq.count())

    # def test_201(self):
    #     self.leak_check(self.body_201)

    # def body_201(self):
    #     self.msgq.insert_tail(gr.message(0))
    #     self.assertEqual(1, self.msgq.count())
    #     self.msgq.insert_tail(gr.message(1))
    #     self.assertEqual(2, self.msgq.count())

    # def test_202(self):
    #     self.leak_check(self.body_202)

    def body_202(self):
        # global msg
        msg = gr.message(666)

    def test_300(self):
        input_data = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
        src = blocks.vector_source_b(input_data)
        dst = blocks.vector_sink_b()
        tb = gr.top_block()
        tb.connect(src, dst)
        tb.run()
        self.assertEqual(input_data, dst.data())

    def test_debug_401(self):
        msg = pmt.intern("TESTING")
        src = blocks.message_strobe(msg, 500)
        snk = blocks.message_debug()

        tb = gr.top_block()
        tb.msg_connect(src, "strobe", snk, "store")
        tb.start()
        time.sleep(1)
        tb.stop()
        tb.wait()

        rec_msg = snk.get_message(0)
        self.assertTrue(pmt.eqv(rec_msg, msg))


if __name__ == '__main__':
    gr_unittest.run(test_message)



#!/usr/bin/env python
#
# Copyright 2012,2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import time
import random
import math

from gnuradio import gr, gr_unittest, filter, blocks


def convolution(A, B):
    """
    Returns a convolution of the A and B vectors of length
    len(A)-len(B).
    """
    rs = []
    for i in range(len(B) - 1, len(A)):
        r = 0
        for j, b in enumerate(B):
            r += A[i - j] * b
        rs.append(r)
    return rs


class test_filterbank_vcvcf(gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        """
        Generates nfilts sets of random complex data.
        Generates two sets of random taps for each filter.
        Applies one set of the random taps, gets some output,
        applies the second set of random taps, gets some more output,
        The output is then compared with a python-implemented
        convolution.
        """
        myrand = random.Random(123).random
        nfilts = 10
        ntaps = 5
        # Sets some of the taps to be all zeros.
        zero_filts1 = (3, 7)
        zero_filts2 = (1, 6, 9)
        ndatapoints = 100
        # Generate some random sets of data
        data_sets = []
        for i in range(0, nfilts):
            data_sets.append([(myrand() - 0.5) + (myrand() - 0.5) * (0 + 1j)
                              for k in range(0, ndatapoints)])
        # Join them together to pass to vector_source block
        data = []
        for dp in zip(*data_sets):
            data += dp
        # Generate some random taps.
        taps1 = []
        taps2 = []
        for i in range(0, nfilts):
            if i in zero_filts1:
                taps1.append([0] * ntaps)
            else:
                taps1.append([myrand() - 0.5 for k in range(0, ntaps)])
            if i in zero_filts2:
                taps2.append([0] * ntaps)
            else:
                taps2.append([myrand() - 0.5 for k in range(0, ntaps)])

        # Calculate results with a python-implemented convolution.
        results = []
        results2 = []
        for ds, ts, ts2 in zip(data_sets, taps1, taps2):
            results.append(convolution(ds[-(len(ts) - 1):] + ds, ts))
            results2.append(convolution(ds[-(len(ts) - 1):] + ds, ts2))
        # Convert results from 2D arrays to 1D arrays for ease of comparison.
        comb_results = []
        for rs in zip(*results):
            comb_results += rs
        comb_results2 = []
        for rs in zip(*results2):
            comb_results2 += rs
        # Construct the signal-processing chain.
        src = blocks.vector_source_c(data, True, nfilts)
        fb = filter.filterbank_vcvcf(taps1)
        v2s = blocks.vector_to_stream(gr.sizeof_gr_complex, nfilts)
        s2v = blocks.stream_to_vector(
            gr.sizeof_gr_complex, nfilts * ndatapoints)
        snk = blocks.probe_signal_vc(nfilts * ndatapoints)
        self.tb.connect(src, fb, v2s, s2v, snk)
        # Run the signal-processing chain.
        self.tb.start()
        all_zero = True
        outdata = None
        waittime = 0.001
        # Wait until we have some data.
        while (not outdata) or outdata[0] == 0:
            time.sleep(waittime)
            outdata = snk.level()
        # Apply the second set of taps.
        fb.set_taps(taps2)
        outdata2 = None
        # Wait until we have new data.
        while (not outdata2) or abs(outdata2[0] - outdata[0]) < 1e-6:
            time.sleep(waittime)
            outdata2 = snk.level()
        self.tb.stop()
        # Compare the datasets.
        self.assertComplexTuplesAlmostEqual(comb_results, outdata, 6)
        self.assertComplexTuplesAlmostEqual(comb_results2, outdata2, 6)


if __name__ == '__main__':
    gr_unittest.run(test_filterbank_vcvcf)



#!/usr/bin/env python
#
# Copyright 2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks

import math


class test_arg_max(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        tb = self.tb

        src1_data = [0, 0.2, -0.3, 0, 12, 0]
        src2_data = [0, 0.0, 3.0, 0, 10, 0]
        src3_data = [0, 0.0, 3.0, 0, 1, 0]

        src1 = blocks.vector_source_f(src1_data)
        s2v1 = blocks.stream_to_vector(gr.sizeof_float, len(src1_data))
        tb.connect(src1, s2v1)

        src2 = blocks.vector_source_f(src2_data)
        s2v2 = blocks.stream_to_vector(gr.sizeof_float, len(src1_data))
        tb.connect(src2, s2v2)

        src3 = blocks.vector_source_f(src3_data)
        s2v3 = blocks.stream_to_vector(gr.sizeof_float, len(src1_data))
        tb.connect(src3, s2v3)

        dst1 = blocks.vector_sink_s()
        dst2 = blocks.vector_sink_s()
        argmax = blocks.argmax_fs(len(src1_data))

        tb.connect(s2v1, (argmax, 0))
        tb.connect(s2v2, (argmax, 1))
        tb.connect(s2v3, (argmax, 2))

        tb.connect((argmax, 0), dst1)
        tb.connect((argmax, 1), dst2)

        tb.run()
        index = dst1.data()
        source = dst2.data()
        self.assertEqual(index, [4, ])
        self.assertEqual(source, [0, ])


if __name__ == '__main__':
    gr_unittest.run(test_arg_max)



#!/usr/bin/env python
#
# Copyright 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import os
import tempfile
import array
import pmt
from gnuradio import gr, gr_unittest, blocks


class test_file_source(gr_unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        os.environ['GR_CONF_CONTROLPORT_ON'] = 'False'
        with tempfile.NamedTemporaryFile(delete=False) as temp:
            cls._datafilename = temp.name
            cls._vector = list(range(1000))
            array.array('f', cls._vector).tofile(temp)

    @classmethod
    def tearDownClass(cls):
        os.unlink(cls._datafilename)

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_file_source(self):
        src = blocks.file_source(gr.sizeof_float, self._datafilename)
        snk = blocks.vector_sink_f()
        self.tb.connect(src, snk)
        self.tb.run()

        result_data = snk.data()
        self.assertFloatTuplesAlmostEqual(self._vector, result_data)
        self.assertEqual(len(snk.tags()), 0)

    def test_file_source_no_such_file(self):
        """
        Try to open a non-existent file and verify exception is thrown.
        """
        with self.assertRaises(RuntimeError):
            blocks.file_source(gr.sizeof_float, "___no_such_file___")

    def test_file_source_with_offset(self):
        expected_result = self._vector[100:]

        src = blocks.file_source(
            gr.sizeof_float,
            self._datafilename,
            offset=100)
        snk = blocks.vector_sink_f()

        self.tb.connect(src, snk)
        self.tb.run()

        result_data = snk.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)
        self.assertEqual(len(snk.tags()), 0)

    def test_source_with_offset_and_len(self):
        expected_result = self._vector[100:100 + 600]

        src = blocks.file_source(
            gr.sizeof_float,
            self._datafilename,
            offset=100,
            len=600)
        snk = blocks.vector_sink_f()
        self.tb.connect(src, snk)
        self.tb.run()

        result_data = snk.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)
        self.assertEqual(len(snk.tags()), 0)

    def test_file_source_can_seek_after_open(self):

        src = blocks.file_source(gr.sizeof_float, self._datafilename)
        self.assertTrue(src.seek(0, os.SEEK_SET))
        self.assertTrue(src.seek(len(self._vector) - 1, os.SEEK_SET))
        # Seek past end of file - this will also log a warning
        self.assertFalse(src.seek(len(self._vector), os.SEEK_SET))
        # Negative seek - this will also log a warning
        self.assertFalse(src.seek(-1, os.SEEK_SET))

        self.assertTrue(src.seek(1, os.SEEK_END))
        self.assertTrue(src.seek(len(self._vector), os.SEEK_END))
        # Seek past end of file - this will also log a warning
        self.assertFalse(src.seek(0, os.SEEK_END))

        self.assertTrue(src.seek(0, os.SEEK_SET))
        self.assertTrue(src.seek(1, os.SEEK_CUR))
        # Seek past end of file - this will also log a warning
        self.assertFalse(src.seek(len(self._vector), os.SEEK_CUR))

    def test_begin_tag(self):
        expected_result = self._vector

        src = blocks.file_source(gr.sizeof_float, self._datafilename)
        src.set_begin_tag(pmt.string_to_symbol("file_begin"))
        snk = blocks.vector_sink_f()
        self.tb.connect(src, snk)
        self.tb.run()

        result_data = snk.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)
        self.assertEqual(len(snk.tags()), 1)

    def test_begin_tag_repeat(self):
        expected_result = self._vector + self._vector

        src = blocks.file_source(gr.sizeof_float, self._datafilename, True)
        src.set_begin_tag(pmt.string_to_symbol("file_begin"))
        head = blocks.head(gr.sizeof_float, 2 * len(self._vector))
        snk = blocks.vector_sink_f()
        self.tb.connect(src, head, snk)
        self.tb.run()

        result_data = snk.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)
        tags = snk.tags()
        self.assertEqual(len(tags), 2)
        self.assertEqual(str(tags[0].key), "file_begin")
        self.assertEqual(str(tags[0].value), "0")
        self.assertEqual(tags[0].offset, 0)
        self.assertEqual(str(tags[1].key), "file_begin")
        self.assertEqual(str(tags[1].value), "1")
        self.assertEqual(tags[1].offset, 1000)


if __name__ == '__main__':
    gr_unittest.run(test_file_source)



#!/usr/bin/env python
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest

import random
import numpy
from gnuradio import digital, blocks, channels


class qa_linear_equalizer(gr_unittest.TestCase):

    def unpack_values(self, values_in, bits_per_value, bits_per_symbol):
        # verify that 8 is divisible by bits_per_symbol
        m = bits_per_value / bits_per_symbol
        # print(m)
        mask = 2**(bits_per_symbol) - 1

        if bits_per_value != m * bits_per_symbol:
            print(
                "error - bits per symbols must fit nicely into bits_per_value bit values")
            return []

        num_values = len(values_in)
        num_symbols = int(num_values * (m))

        cur_byte = 0
        cur_bit = 0
        out = []
        for i in range(num_symbols):
            s = (
                values_in[cur_byte] >> (
                    bits_per_value -
                    bits_per_symbol -
                    cur_bit)) & mask
            out.append(s)
            cur_bit += bits_per_symbol

            if cur_bit >= bits_per_value:
                cur_bit = 0
                cur_byte += 1

        return out

    def map_symbols_to_constellation(self, symbols, cons):
        l = list(map(lambda x: cons.points()[x], symbols))
        return l

    def setUp(self):
        random.seed(987654)
        self.tb = gr.top_block()
        self.num_data = num_data = 10000

        self.sps = sps = 4
        self.eb = eb = 0.35
        self.preamble = preamble = [
            0x27,
            0x2F,
            0x18,
            0x5D,
            0x5B,
            0x2A,
            0x3F,
            0x71,
            0x63,
            0x3C,
            0x17,
            0x0C,
            0x0A,
            0x41,
            0xD6,
            0x1F,
            0x4C,
            0x23,
            0x65,
            0x68,
            0xED,
            0x1C,
            0x77,
            0xA7,
            0x0E,
            0x0A,
            0x9E,
            0x47,
            0x82,
            0xA4,
            0x57,
            0x24,
        ]

        self.payload_size = payload_size = 300  # bytes
        self.data = data = [0] * 4 + \
            [random.getrandbits(8) for i in range(payload_size)]
        self.gain = gain = .001  # LMS gain
        self.corr_thresh = corr_thresh = 3e6
        self.num_taps = num_taps = 16

    def tearDown(self):
        self.tb = None

    def transform(self, src_data, const, alg):
        src = blocks.vector_source_c(src_data, False)
        leq = digital.linear_equalizer(
            4,
            1,
            alg,
            True,
            [],
            '')
        dst = blocks.vector_sink_c()
        self.tb.connect(src, leq, dst)
        self.tb.run()
        return dst.data()

    def test_001_identity_lms(self):
        # Constant modulus signal so no adjustments
        const = digital.constellation_qpsk()
        src_data = const.points() * 1000
        alg = digital.adaptive_algorithm_lms(const, .1).base()

        N = 100  # settling time
        expected_data = src_data[N:]
        result = self.transform(src_data, const, alg)[N:]

        N = -500
        self.assertComplexTuplesAlmostEqual(expected_data[N:], result[N:], 5)

    def test_002_identity_cma(self):
        # Constant modulus signal so no adjustments
        const = digital.constellation_qpsk()
        src_data = const.points() * 1000
        alg = digital.adaptive_algorithm_cma(const, .001, 4).base()

        N = 100  # settling time
        expected_data = src_data[N:]
        result = self.transform(src_data, const, alg)[N:]

        N = -500
        self.assertComplexTuplesAlmostEqual(expected_data[N:], result[N:], 5)

    def test_qpsk_3tap_lms_training(self):
        # set up fg
        gain = 0.01  # LMS gain
        num_taps = 16
        num_samp = 2000
        num_test = 500
        cons = digital.constellation_qpsk().base()
        rxmod = digital.generic_mod(
            cons, False, self.sps, True, self.eb, False, False)
        modulated_sync_word_pre = digital.modulate_vector_bc(
            rxmod.to_basic_block(), self.preamble + self.preamble, [1])
        # compensate for the RRC filter delay
        modulated_sync_word = modulated_sync_word_pre[86:(512 + 86)]
        corr_max = numpy.abs(
            numpy.dot(
                modulated_sync_word,
                numpy.conj(modulated_sync_word)))
        corr_calc = self.corr_thresh / (corr_max * corr_max)
        preamble_symbols = self.map_symbols_to_constellation(
            self.unpack_values(self.preamble, 8, 2), cons)

        alg = digital.adaptive_algorithm_lms(cons, gain).base()
        evm = digital.meas_evm_cc(cons, digital.evm_measurement_t.EVM_PERCENT)
        leq = digital.linear_equalizer(
            num_taps,
            self.sps,
            alg,
            False,
            preamble_symbols,
            'corr_est')
        correst = digital.corr_est_cc(
            modulated_sync_word,
            self.sps,
            12,
            corr_calc,
            digital.THRESHOLD_ABSOLUTE)
        constmod = digital.generic_mod(
            constellation=cons,
            differential=False,
            samples_per_symbol=4,
            pre_diff_code=True,
            excess_bw=0.35,
            verbose=False,
            log=False)
        chan = channels.channel_model(
            noise_voltage=0.0,
            frequency_offset=0.0,
            epsilon=1.0,
            taps=(1.0 + 1.0j, 0.63 - .22j, -.1 + .07j),
            noise_seed=0,
            block_tags=False)
        vso = blocks.vector_source_b(self.preamble + self.data, True, 1, [])
        head = blocks.head(gr.sizeof_float * 1, num_samp)
        vsi = blocks.vector_sink_f()

        self.tb.connect(vso, constmod, chan, correst, leq, evm, head, vsi)
        self.tb.run()

        # look at the last 1000 samples, should converge quickly, below 5% EVM
        upper_bound = list(20.0 * numpy.ones((num_test,)))
        lower_bound = list(0.0 * numpy.zeros((num_test,)))
        output_data = vsi.data()
        output_data = output_data[-num_test:]
        self.assertLess(output_data, upper_bound)
        self.assertGreater(output_data, lower_bound)


if __name__ == '__main__':
    gr_unittest.run(qa_linear_equalizer)



"""
Copyright 2010-2011,2014,2018 Free Software Foundation, Inc.

This file is part of GNU Radio

SPDX-License-Identifier: GPL-2.0-or-later

"""

import sys

MAIN_TMPL = """\
id: uhd_usrp_${sourk}
label: 'UHD: USRP ${sourk.title()}'
flags: [python, cpp, throttle]

parameters:
-   id: type
    label: ${direction.title()}put Type
    dtype: enum
    options: [fc32, sc16, item32]
    option_labels: [Complex float32, Complex int16, VITA word32]
    option_attributes:
        type: [fc32, sc16, s32]
    hide: part
-   id: otw
    label: Wire Format
    dtype: enum
    options: ['', sc16, sc12, sc8]
    option_labels: [Automatic, Complex int16, Complex int12, Complex int8]
    hide: ${'$'}{ 'none' if otw else 'part'}
-   id: stream_args
    label: Stream args
    dtype: string
    options: ['', peak=0.003906]
    option_labels: ['', peak=0.003906]
    hide: ${'$'}{ 'none' if stream_args else 'part'}
-   id: stream_chans
    label: Stream channels
    dtype: int_vector
    default: '[]'
    hide: ${'$'}{ 'none' if stream_chans else 'part'}
-   id: dev_addr
    label: Device Address
    dtype: string
    default: '""'
    hide: ${'$'}{ 'none' if dev_addr else 'part'}
-   id: dev_args
    label: Device Arguments
    dtype: string
    hide: ${'$'}{ 'none' if dev_args else 'part'}
-   id: sync
    label: Sync
    dtype: enum
    options: [sync, pc_clock, pc_clock_next_pps, gps_time, none]
    option_labels: [Unknown PPS, PC Clock, PC Clock on Next PPS, GPS Time on Next PPS, No Sync]
    hide: ${'$'}{ 'none' if sync else 'part'}
-   id: start_time
    label: Start Time (seconds)
    dtype: real
    default: -1.0
    options: [-1.0]
    option_labels: [Default]
    hide: ${'$'}{ 'none' if start_time >= 0.0 else 'part' }
-   id: clock_rate
    label: Clock Rate (Hz)
    dtype: real
    default: 0e0
    options: [0e0, 200e6, 184.32e6, 153.6e6, 125.0e6, 122.88e6, 120e6, 30.72e6]
    option_labels: [Default, 200 MHz, 184.32 MHz, 153.6 MHz, 125 MHz, 122.88 MHz, 120 MHz, 30.72 MHz]
    hide: ${'$'}{ 'none' if clock_rate else 'part' }
-   id: num_mboards
    label: Num Mboards
    dtype: int
    default: 1
    options: [1, 2, 3, 4, 5, 6, 7, 8]
    hide: part
% for m in range(max_mboards):
-   id: clock_source${m}
    label: 'Mb${m}: Clock Source'
    dtype: string
    options: ['', internal, external, mimo, gpsdo]
    option_labels: [Default, Internal, External, MIMO Cable, O/B GPSDO]
    hide: ${'$'}{ 'all' if not (num_mboards > ${m}) else ( 'none' if clock_source${m} else 'part')}
-   id: time_source${m}
    label: 'Mb${m}: Time Source'
    dtype: string
    options: ['', external, mimo, gpsdo]
    option_labels: [Default, External, MIMO Cable, O/B GPSDO]
    hide: ${'$'}{ 'all' if not (num_mboards > ${m}) else ('none' if time_source${m} else 'part')}
-   id: sd_spec${m}
    label: 'Mb${m}: Subdev Spec'
    dtype: string
    hide: ${'$'}{ 'all' if not (num_mboards > ${m}) else ('none' if sd_spec${m} else 'part')}
% endfor
-   id: nchan
    label: Num Channels
    dtype: int
    default: 1
    options: [ ${", ".join([str(n) for n in range(1, max_nchan+1)])} ]
    hide: part
-   id: samp_rate
    label: Samp rate (Sps)
    dtype: real
    default: samp_rate
${params}

inputs:
-   domain: message
    id: command
    optional: true
% if sourk == 'source':

outputs:
% endif
-   domain: stream
    dtype: ${'$'}{type.type}
    multiplicity: ${'$'}{nchan}
% if sourk == 'sink':

outputs:
% endif
-   domain: message
    id: async_msgs
    optional: true

templates:
    imports: |-
        from gnuradio import uhd
        import time
    make: |
        uhd.usrp_${sourk}(
            ${'%'} if clock_rate():
            ",".join((${'$'}{dev_addr}, ${'$'}{dev_args}, "master_clock_rate=${'$'}{clock_rate}")),
            ${'%'} else:
            ",".join((${'$'}{dev_addr}, ${'$'}{dev_args})),
            ${'%'} endif
            uhd.stream_args(
                cpu_format="${'$'}{type}",
                ${'%'} if otw:
                otw_format="${'$'}{otw}",
                ${'%'} endif
                ${'%'} if stream_args:
                args=${'$'}{stream_args},
                ${'%'} endif
                ${'%'} if eval(stream_chans):
                channels=${'$'}{stream_chans},
                ${'%'} else:
                channels=list(range(0,${'$'}{nchan})),
                ${'%'} endif
            ),
            % if sourk == 'sink':
            ${'%'} if len_tag_name:
            ${'$'}{len_tag_name},
            ${'%'} endif
            % endif
        )
        % for m in range(max_mboards):
        ${'%'} if context.get('num_mboards')() > ${m}:
        ########################################################################
        ${'%'} if context.get('clock_source${m}')():
        self.${'$'}{id}.set_clock_source(${'$'}{${'clock_source' + str(m)}}, ${m})
        ${'%'} endif
        ########################################################################
        ${'%'} if context.get('time_source${m}')():
        self.${'$'}{id}.set_time_source(${'$'}{${'time_source' + str(m)}}, ${m})
        ${'%'} endif
        ########################################################################
        ${'%'} if context.get('sd_spec${m}')():
        self.${'$'}{id}.set_subdev_spec(${'$'}{${'sd_spec' + str(m)}}, ${m})
        ${'%'} endif
        ########################################################################
        ${'%'} endif
        % endfor  # for m in range(max_mboards)
        self.${'$'}{id}.set_samp_rate(${'$'}{samp_rate})
        ${'%'} if sync == 'sync':
        self.${'$'}{id}.set_time_unknown_pps(uhd.time_spec(0))
        ${'%'} elif sync == 'pc_clock':
        self.${'$'}{id}.set_time_now(uhd.time_spec(time.time()), uhd.ALL_MBOARDS)
        ${'%'} elif sync == 'pc_clock_next_pps':
        _last_pps_time = self.${'$'}{id}.get_time_last_pps().get_real_secs()
        # Poll get_time_last_pps() every 50 ms until a change is seen
        while(self.${'$'}{id}.get_time_last_pps().get_real_secs() == _last_pps_time):
            time.sleep(0.05)
        # Set the time to PC time on next PPS
        self.${'$'}{id}.set_time_next_pps(uhd.time_spec(int(time.time()) + 1.0))
        # Sleep 1 second to ensure next PPS has come
        time.sleep(1)
        ${'%'} elif sync == 'gps_time':
        # Set the time to GPS time on next PPS
        # get_mboard_sensor("gps_time") returns just after the PPS edge,
        # thus add one second and set the time on the next PPS
        self.${'$'}{id}.set_time_next_pps(uhd.time_spec(self.${'$'}{id}.get_mboard_sensor("gps_time").to_int() + 1.0))
        # Sleep 1 second to ensure next PPS has come
        time.sleep(1)
        ${'%'} else:
        # No synchronization enforced.
        ${'%'} endif

        % for n in range(max_nchan):
        ${'%'} if context.get('nchan')() > ${n}:
        self.${'$'}{id}.set_center_freq(${'$'}{${'center_freq' + str(n)}}, ${n})
        ${'%'} if context.get('ant${n}')():
        self.${'$'}{id}.set_antenna(${'$'}{${'ant' + str(n)}}, ${n})
        ${'%'} endif
        ${'%'} if context.get('bw${n}')():
        self.${'$'}{id}.set_bandwidth(${'$'}{${'bw' + str(n)}}, ${n})
        ${'%'} endif
        % if sourk == 'source':
        ${'%'} if context.get('rx_agc${n}')() == 'Enabled':
        self.${'$'}{id}.set_rx_agc(True, ${n})
        ${'%'} elif context.get('rx_agc${n}')() == 'Disabled':
        self.${'$'}{id}.set_rx_agc(False, ${n})
        ${'%'} endif
        ${'%'} if context.get('rx_agc${n}')() != 'Enabled':
        ${'%'} if context.get('gain_type' + '${n}')() == 'normalized':
        self.${'$'}{id}.set_normalized_gain(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} elif context.get('gain_type' + '${n}')() == 'power':
        self.${'$'}{id}.set_power_reference(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} else:
        self.${'$'}{id}.set_gain(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} endif
        ${'%'} endif  # if rx_agc${n} != 'Enabled'
        ${'%'} if context.get('dc_offs_enb${n}')() in ('auto', 'disabled'):
        self.${'$'}{id}.set_auto_dc_offset(${'$'}{True if ${'dc_offs_enb' + str(n)} == 'auto' else False}, ${n})
        ${'%'} elif context.get('dc_offs_enb${n}')() == 'manual':
        self.${'$'}{id}.set_dc_offset(${'$'}{${'dc_offs' + str(n)}}, ${n})
        ${'%'} endif
        ${'%'} if context.get('iq_imbal_enb${n}')() in ('auto', 'disabled'):
        self.${'$'}{id}.set_auto_iq_balance(${'$'}{True if ${'iq_imbal_enb' + str(n)} == 'auto' else False}, ${n})
        ${'%'} elif context.get('iq_imbal_enb${n}')() == 'manual':
        self.${'$'}{id}.set_iq_balance(${'$'}{${'iq_imbal' + str(n)}}, ${n})
        ${'%'} endif
        % else:
        ${'%'} if context.get('gain_type' + '${n}')() == 'normalized':
        self.${'$'}{id}.set_normalized_gain(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} elif context.get('gain_type' + '${n}')() == 'power':
        self.${'$'}{id}.set_power_reference(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} else:
        self.${'$'}{id}.set_gain(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} endif
        % endif  # if sourk == 'source'

        ${'%'} if context.get('show_lo_controls')():
        self.${'$'}{id}.set_lo_source(${'$'}{${'lo_source' + str(n)}}, uhd.ALL_LOS, ${n})
        self.${'$'}{id}.set_lo_export_enabled(${'$'}{${'lo_export' + str(n)}}, uhd.ALL_LOS, ${n})
        ${'%'} endif
        ${'%'} endif  # nchan > n
        % endfor  # for n in range(max_nchan)
        ${'%'} if start_time() >= 0.0:
        self.${'$'}{id}.set_start_time(uhd.time_spec(${'$'}{start_time}))
        ${'%'} endif
    callbacks:
    -   set_samp_rate(${'$'}{samp_rate})
    % for n in range(max_nchan):
    -   set_center_freq(${'$'}{${'center_freq' + str(n)}}, ${n})
    % if sourk == 'source':
    -   ${'$'}{'set_rx_agc(True, ${n})' if context.get('rx_agc${n}')() == 'Enabled' else ''}
    -   ${'$'}{'set_rx_agc(False, ${n})' if context.get('rx_agc${n}')() == 'Disabled' else ''}
    -   |
        ${'%'} if context.get('rx_agc${n}')() != 'Enabled':
        ${'%'} if context.get('gain_type' + '${n}')() == 'normalized':
        self.${'$'}{id}.set_normalized_gain(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} elif context.get('gain_type' + '${n}')() == 'power':
        self.${'$'}{id}.set_power_reference(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} else:
        self.${'$'}{id}.set_gain(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} endif
        ${'%'} endif
    % else:
    -   |
        ${'%'} if context.get('gain_type' + '${n}')() == 'normalized':
        self.${'$'}{id}.set_normalized_gain(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} elif context.get('gain_type' + '${n}')() == 'power':
        self.${'$'}{id}.set_power_reference(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} else:
        self.${'$'}{id}.set_gain(${'$'}{${'gain' + str(n)}}, ${n})
        ${'%'} endif
    % endif
    -   ${'$'}{'set_lo_source(' + lo_source${n} + ', uhd.ALL_LOS, ${n})' if show_lo_controls else ''}
    -   ${'$'}{'set_lo_export_enabled(' + lo_export${n} + ', uhd.ALL_LOS, ${n})' if show_lo_controls else ''}
    -   set_antenna(${'$'}{${'ant' + str(n)}}, ${n})
    -   set_bandwidth(${'$'}{${'bw' + str(n)}}, ${n})
    % endfor

cpp_templates:
    includes: [ '#include <gnuradio/uhd/usrp_${sourk}.h>' ]
    declarations: 'gr::uhd::usrp_${sourk}::sptr ${'$'}{id};'
    make: |
      this->${'$'}{id} = gr::uhd::usrp_${sourk}::make(
         ::uhd::device_addr_t("${'$'}{",".join((str(dev_addr).strip('"\\''), str(dev_args).strip('"\\''))) if len(str(dev_args).strip('"\\'')) > 0 else dev_addr.strip('"\\'')}"),
         ::uhd::stream_args_t("${'$'}{type}", "${'$'}{otw}"));
      % for m in range(max_mboards):
      ${'%'} if context.get('num_mboards')() > ${m}:
      ${'%'} if context.get('sd_spec${m}')():
      this->${'$'}{id}->set_subdev_spec(${'$'}{${'sd_spec' + str(m)}}, ${m});
      ${'%'} endif
      ${'%'} if context.get('time_source${m}')():
      this->${'$'}{id}->set_time_source(${'$'}{${'time_source' + str(m)}}, ${m});
      ${'%'} endif
      ${'%'} if context.get('clock_source${m}')():
      this->${'$'}{id}->set_clock_source("${'$'}{${'clock_source' + str(m)}.strip('\\'')}", ${m});
      ${'%'} endif
      ${'%'} endif
      % endfor
      this->${'$'}{id}->set_samp_rate(${'$'}{samp_rate});
      ${'%'} if sync == 'sync':
      this->${'$'}{id}->set_time_unknown_pps(::uhd::time_spec_t());
      ${'%'} elif sync == 'pc_clock':
      this->${'$'}{id}->set_time_now(::uhd::time_spec_t(time(NULL)), ::uhd::usrp::multi_usrp::ALL_MBOARDS);
      ${'%'} else:
      // No synchronization enforced.
      ${'%'} endif

      % for n in range(max_nchan):
      ${'%'} if context.get('nchan')() > ${n}:
      this->${'$'}{id}->set_center_freq(${'$'}{${'center_freq' + str(n)}}, ${n});
      % if sourk == 'source':
      ${'%'} if context.get('rx_agc${n}')() == 'Enabled':
      this->${'$'}{id}->set_rx_agc(True, ${n});
      ${'%'} elif context.get('rx_agc${n}')() == 'Disabled':
      this->${'$'}{id}->set_rx_agc(False, ${n});
      ${'%'} endif
      ${'%'} if context.get('rx_agc${n}')() != 'Enabled':
      ${'%'} if context.get('gain_type' + '${n}')() == 'normalized':
      this->${'$'}{id}->set_normalized_gain(${'$'}{${'gain' + str(n)}}, ${n});
      ${'%'} elif context.get('gain_type' + '${n}')() == 'power':
      this->${'$'}{id}->set_power_reference(${'$'}{${'gain' + str(n)}}, ${n});
      ${'%'} else:
      this->${'$'}{id}->set_gain(${'$'}{${'gain' + str(n)}}, ${n});
      ${'%'} endif
      ${'%'} endif
      % else:
      ${'%'} if context.get('gain_type' + '${n}')() == 'normalized':
      this->${'$'}{id}->set_normalized_gain(${'$'}{${'gain' + str(n)}}, ${n});
      ${'%'} elif context.get('gain_type' + '${n}')() == 'power':
      this->${'$'}{id}->set_power_reference(${'$'}{${'gain' + str(n)}}, ${n});
      ${'%'} else:
      this->${'$'}{id}->set_gain(${'$'}{${'gain' + str(n)}}, ${n});
      ${'%'} endif
      % endif
      ${'%'} if context.get('ant${n}')():
      this->${'$'}{id}->set_antenna(${'$'}{${'ant' + str(n)}}, ${n});
      ${'%'} endif
      ${'%'} if context.get('bw${n}')():
      this->${'$'}{id}->set_bandwidth(${'$'}{${'bw' + str(n)}}, ${n});
      ${'%'} endif
      ${'%'} if context.get('show_lo_controls')():
      this->${'$'}{id}->set_lo_source(${'$'}{${'lo_source' + str(n)}}, ::uhd::usrp::multi_usrp::ALL_LOS, ${n});
      this->${'$'}{id}->set_lo_export_enabled(${'$'}{${'lo_export' + str(n)}}, ::uhd::usrp::multi_usrp::ALL_LOS, ${n});
      ${'%'} endif
      ${'%'} endif
      % endfor
      ${'%'} if clock_rate():
      this->${'$'}{id}->set_clock_rate(${'$'}{clock_rate}, ::uhd::usrp::multi_usrp::ALL_MBOARDS);
      ${'%'} endif
      ${'%'} if start_time() >= 0.0:
      this->${'$'}{id}->set_start_time(::uhd::time_spec_t(${'$'}{float(start_time)}));
      ${'%'} endif
    link: ['gnuradio-uhd uhd']
    callbacks:
    - set_samp_rate(${'$'}{samp_rate})
    % for n in range(max_nchan):
    -   set_center_freq(${'$'}{${'center_freq' + str(n)}}, ${n})
    % if sourk == 'source':
    -   ${'$'}{'set_rx_agc(True, ${n})' if context.get('rx_agc${n}')() == 'Enabled' else ''}
    -   ${'$'}{'set_rx_agc(False, ${n})' if context.get('rx_agc${n}')() == 'Disabled' else ''}
    -   |
        ${'%'} if context.get('rx_agc${n}')() != 'Enabled':
        ${'%'} if context.get('gain_type' + '${n}')() == 'normalized':
        this->${'$'}{id}->set_normalized_gain(${'$'}{${'gain' + str(n)}}, ${n});
        ${'%'} elif context.get('gain_type' + '${n}')() == 'power':
        this->${'$'}{id}->set_power_reference(${'$'}{${'gain' + str(n)}}, ${n});
        ${'%'} else:
        this->${'$'}{id}->set_gain(${'$'}{${'gain' + str(n)}}, ${n});
        ${'%'} endif
        ${'%'} endif
    % else:
    -   |
        ${'%'} if context.get('gain_type' + '${n}')() == 'normalized':
        this->${'$'}{id}->set_normalized_gain(${'$'}{${'gain' + str(n)}}, ${n});
        ${'%'} elif context.get('gain_type' + '${n}')() == 'power':
        this->${'$'}{id}->set_power_reference(${'$'}{${'gain' + str(n)}}, ${n});
        ${'%'} else:
        this->${'$'}{id}->set_gain(${'$'}{${'gain' + str(n)}}, ${n});
        ${'%'} endif
    % endif
    -   ${'$'}{'set_lo_source(' + lo_source${n} + ', ::uhd::usrp::multi_usrp::ALL_LOS, ${n})' if show_lo_controls else ''}
    -   ${'$'}{'set_lo_export_enabled(' + lo_export${n} + ', ::uhd::usrp::multi_usrp::ALL_LOS, ${n})' if show_lo_controls else ''}
    -   set_antenna(${'$'}{${'ant' + str(n)}}, ${n})
    -   set_bandwidth(${'$'}{${'bw' + str(n)}}, ${n})
    % endfor

documentation: |-
    The UHD USRP ${sourk.title()} Block:

    Device Address:
    The device address is a delimited string used to locate UHD devices on your system. \\
    If left blank, the first UHD device found will be used. \\
    Use the device address to specify a specific device or list of devices.
    USRP1 Example: serial=12345678
    USRP2 Example: addr=192.168.10.2
    USRP2 Example: addr0=192.168.10.2, addr1=192.168.10.3

    ${direction.title()} Type:
    This parameter controls the data type of the stream in gnuradio.

    Wire Format:
    This parameter controls the form of the data over the bus/network. \
    Complex bytes may be used to trade off precision for bandwidth. \
    Not all formats are supported on all devices.

    Stream Args:
    Optional arguments to be passed in the UHD streamer object. \
    Streamer args is a list of key/value pairs; usage is determined by the implementation.
    Ex: the scalar key affects the scaling between 16 and 8 bit integers in sc8 wire format.

    Num Motherboards:
    Selects the number of USRP motherboards in this device configuration.

    Reference Source:
    Where the motherboard should sync its time and clock references.
    If source and sink blocks reference the same device,
    it is only necessary to set the reference source on one of the blocks.

    Subdevice specification:
    Each motherboard should have its own subdevice specification \\
    and all subdevice specifications should be the same length. \\
    Select the subdevice or subdevices for each channel using a markup string. \\
    The markup string consists of a list of dboard_slot:subdev_name pairs (one pair per channel). \\
    If left blank, the UHD will try to select the first subdevice on your system. \\
    See the application notes for further details.
    Single channel example: :AB
    Dual channel example: :A :B

    Num Channels:
    Selects the total number of channels in this multi-USRP configuration.
    Ex: 4 motherboards with 2 channels per board = 8 channels total

    Sample rate:
    The sample rate is the number of samples per second input by this  block. \\
    The UHD device driver will try its best to match the requested sample rate. \\
    If the requested rate is not possible, the UHD block will print an error at runtime.

    Center frequency:
    The center frequency is the overall frequency of the RF chain. \\
    For greater control of how the UHD tunes elements in the RF chain, \\
    pass a tune_request object rather than a simple target frequency.
    Tuning with an LO offset example: uhd.tune_request(freq, lo_off)
    Tuning without DSP: uhd.tune_request(target_freq, dsp_freq=0, \\
    dsp_freq_policy=uhd.tune_request.POLICY_MANUAL)

    Antenna:
    For subdevices with only one antenna, this may be left blank. \\
    Otherwise, the user should specify one of the possible antenna choices. \\
    See the daughterboard application notes for the possible antenna choices.

    Bandwidth:
    To use the default bandwidth filter setting, this should be zero. \\
    Only certain subdevices have configurable bandwidth filters. \\
    See the daughterboard application notes for possible configurations.

    Length tag key (Sink only):
    When a nonempty string is given, the USRP sink will look for length tags \\
    to determine transmit burst lengths.

    See the UHD manual for more detailed documentation:
    http://uhd.ettus.com

file_format: 1
"""

PARAMS_TMPL = """
-   id: center_freq${n}
    label: 'Ch${n}: Center Freq (Hz)'
    category: RF Options
    dtype: raw
    default: '0'
    hide: ${'$'}{ 'none' if (nchan > ${n}) else 'all' }
% if sourk == 'source':
-   id: rx_agc${n}
    label: 'Ch${n}: AGC'
    category: RF Options
    dtype: string
    default: 'Default'
    options: ['Default', 'Disabled', 'Enabled']
    option_labels: [Default, Disabled, Enabled]
    hide: ${'$'}{ 'none' if (nchan > ${n}) else 'all' }
% endif
-   id: gain${n}
    label: 'Ch${n}: Gain Value'
    category: RF Options
    dtype: float
    default: '0'
% if sourk == 'source':
    hide: ${'$'}{ 'none' if nchan > ${n} and rx_agc${n} != 'Enabled' else 'all' }
% else:
    hide: ${'$'}{ 'none' if nchan > ${n} else 'all' }
% endif
-   id: gain_type${n}
    label: 'Ch${n}: Gain Type'
    category: RF Options
    dtype: enum
    options: [default, normalized, power]
    option_labels: [Absolute (dB), Normalized, Absolute Power (dBm)]
% if sourk == 'source':
    hide: ${'$'}{ 'all' if nchan <= ${n} or rx_agc${n} == 'Enabled' else ('part' if (eval('gain_type' + str(${n})) == 'default') else 'none')}
% else:
    hide: ${'$'}{ 'all' if nchan <= ${n} else ('part' if (eval('gain_type' + str(${n})) == 'default') else 'none')}
% endif
-   id: ant${n}
    label: 'Ch${n}: Antenna'
    category: RF Options
    dtype: string
% if sourk == 'source':
    options: ['"TX/RX"', '"RX2"', '"RX1"']
    option_labels: [TX/RX, RX2, RX1]
    default: '"RX2"'
% else:
    options: ['"TX/RX"']
    option_labels: [TX/RX]
% endif
    hide: ${'$'}{ 'all' if not nchan > ${n} else ('none' if eval('ant' + str(${n})) else 'part')}
-   id: bw${n}
    label: 'Ch${n}: Bandwidth (Hz)'
    category: RF Options
    dtype: real
    default: '0'
    hide: ${'$'}{ 'all' if not nchan > ${n} else ('none' if eval('bw' + str(${n})) else 'part')}
-   id: lo_source${n}
    label: 'Ch${n}: LO Source'
    category: RF Options
    dtype: string
    default: internal
    options: [internal, external, companion]
    hide: ${'$'}{ 'all' if not nchan > ${n} else ('none' if show_lo_controls else 'all')}
-   id: lo_export${n}
    label: 'Ch${n}: LO Export'
    category: RF Options
    dtype: bool
    default: 'False'
    options: ['True', 'False']
    hide: ${'$'}{ 'all' if not nchan > ${n} else ('none' if show_lo_controls else 'all')}
% if sourk == 'source':
-   id: dc_offs_enb${n}
    label: 'Ch${n}: Enable DC Offset Correction'
    category: FE Corrections
    dtype: enum
    options: [default, auto, disabled, manual]
    option_labels: [Default, Automatic, Disabled, Manual]
    hide: ${'$'}{ 'all' if not nchan > ${n} else 'part'}
-   id: dc_offs${n}
    label: 'Ch${n}: DC Offset Correction Value'
    category: FE Corrections
    dtype: complex
    default: 0+0j
    hide: ${'$'}{ 'all' if not dc_offs_enb${n} == 'manual' else 'part'}
-   id: iq_imbal_enb${n}
    label: 'Ch${n}: Enable IQ Imbalance Correction'
    category: FE Corrections
    dtype: enum
    options: [default, auto, disabled, manual]
    option_labels: [Default, Automatic, Disabled, Manual]
    hide: ${'$'}{ 'all' if not nchan > ${n} else 'part'}
-   id: iq_imbal${n}
    label: 'Ch${n}: IQ imbalance Correction Value'
    category: FE Corrections
    dtype: complex
    default: 0+0j
    hide: ${'$'}{ 'all' if not iq_imbal_enb${n} == 'manual' else 'part'}
% endif
"""

SHOW_LO_CONTROLS_PARAM = """
-   id: show_lo_controls
    label: Show LO Controls
    category: Advanced
    dtype: bool
    default: 'False'
    hide: part
"""

TSBTAG_PARAM = """
-   id: len_tag_name
    label: TSB tag name
    dtype: string
    default: '""'
    hide: ${ 'none' if len(str(len_tag_name)) else 'part'}
"""

TSBTAG_ARG = """
${'%'} if len_tag_name():
${'$'}{len_tag_name},
${'%'} endif
"""


def parse_tmpl(_tmpl, **kwargs):
    """ Render _tmpl using the kwargs. """
    from mako.template import Template
    block_template = Template(_tmpl)
    return str(block_template.render(**kwargs))


MAX_NUM_MBOARDS = 8
MAX_NUM_CHANNELS = MAX_NUM_MBOARDS * 4

if __name__ == '__main__':
    for file in sys.argv[1:]:
        if file.endswith('source.block.yml'):
            sourk = 'source'
            direction = 'out'
        elif file.endswith('sink.block.yml'):
            sourk = 'sink'
            direction = 'in'
        else:
            raise Exception('is % a source or sink?' % file)
        params = ''.join([
            parse_tmpl(PARAMS_TMPL, n=n, sourk=sourk)
            for n in range(MAX_NUM_CHANNELS)
        ])
        params += SHOW_LO_CONTROLS_PARAM
        if sourk == 'sink':
            params += TSBTAG_PARAM
            lentag_arg = TSBTAG_ARG
        else:
            lentag_arg = ''
        open(file, 'w').write(
            parse_tmpl(
                MAIN_TMPL,
                lentag_arg=lentag_arg,
                max_nchan=MAX_NUM_CHANNELS,
                max_mboards=MAX_NUM_MBOARDS,
                params=params,
                sourk=sourk,
                direction=direction,
            )
        )



#!/usr/bin/env python
#
# Copyright 2011,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import random
import math

import pmt

from gnuradio import gr, gr_unittest, filter, analog, blocks, digital
from gnuradio.digital.utils import mod_codes, alignment
from gnuradio.digital import packet_utils
from gnuradio.digital.generic_mod_demod import generic_mod, generic_demod

from qa_constellation import tested_constellations, twod_constell

# Set a seed so that if errors turn up they are reproducible.
SEED = 1239

# TESTING PARAMETERS
# The number of symbols to test with.
# We need this many to let the frequency recovery block converge.
DATA_LENGTH = 1000
# Test fails if fraction of output that is correct is less than this.
EASY_REQ_CORRECT = 0.9
# For constellations that aren't expected to work so well.
MEDIUM_REQ_CORRECT = 0.8

# CHANNEL PARAMETERS
NOISE_VOLTAGE = 0.01
FREQUENCY_OFFSET = 0.01
TIMING_OFFSET = 1.0

# RECEIVER PARAMETERS
FREQ_BW = 2 * math.pi / 100.0
PHASE_BW = 2 * math.pi / 100.0


class channel_model(gr.hier_block2):
    def __init__(self, noise_voltage, freq, timing):
        gr.hier_block2.__init__(self, "channel_model",
                                gr.io_signature(1, 1, gr.sizeof_gr_complex),
                                gr.io_signature(1, 1, gr.sizeof_gr_complex))

        timing_offset = filter.mmse_resampler_cc(0, timing)
        noise_adder = blocks.add_cc()
        noise = analog.noise_source_c(analog.GR_GAUSSIAN,
                                      noise_voltage, 0)
        freq_offset = analog.sig_source_c(1, analog.GR_SIN_WAVE,
                                          freq, 1.0, 0.0)
        mixer_offset = blocks.multiply_cc()

        self.connect(self, timing_offset)
        self.connect(timing_offset, (mixer_offset, 0))
        self.connect(freq_offset, (mixer_offset, 1))
        self.connect(mixer_offset, (noise_adder, 1))
        self.connect(noise, (noise_adder, 0))
        self.connect(noise_adder, self)


class test_constellation_receiver(gr_unittest.TestCase):

    # We ignore the first half of the output data since often it takes
    # a while for the receiver to lock on.
    ignore_fraction = 0.8
    max_data_length = DATA_LENGTH * 6
    max_num_samples = 1000

    def setUp(self):
        random.seed(0)

    def test_basic(self):
        """
        Tests a bunch of different constellations by using generic
        modulation, a channel, and generic demodulation.  The generic
        demodulation uses constellation_receiver which is what
        we're really trying to test.
        """

        rndm = random.Random()
        rndm.seed(SEED)
        # Assumes not more than 64 points in a constellation
        # Generates some random input data to use.
        self.src_data = tuple(
            [rndm.randint(0, 1) for i in range(0, self.max_data_length)])
        # Generates some random indices to use for comparing input and
        # output data (a full comparison is too slow in python).
        self.indices = alignment.random_sample(
            self.max_data_length, self.max_num_samples, SEED)

        requirements = (
            (EASY_REQ_CORRECT,
             tested_constellations(
                 easy=True,
                 medium=False,
                 difficult=False)),
            (MEDIUM_REQ_CORRECT,
             tested_constellations(
                 easy=False,
                 medium=True,
                 difficult=False)),
        )
        for req_correct, tcs in requirements:
            for constellation, differential in tcs:
                # The constellation_receiver doesn't work for constellations
                # of multiple dimensions (i.e. multiple complex numbers to a
                # single symbol).
                # That is not implemented since the receiver has no way of
                # knowing where the beginning of a symbol is.
                # It also doesn't work for non-differential modulation.
                if constellation.dimensionality() != 1 or not differential:
                    continue
                data_length = DATA_LENGTH * constellation.bits_per_symbol()
                tb = rec_test_tb(constellation, differential,
                                 src_data=self.src_data[:data_length])
                tb.run()
                data = tb.dst.data()
                d1 = tb.src_data[:int(len(tb.src_data) * self.ignore_fraction)]
                d2 = data[:int(len(data) * self.ignore_fraction)]
                correct, overlap, offset, indices = alignment.align_sequences(
                    d1, d2, indices=self.indices)
                self.assertGreater(correct, req_correct,
                                   msg=f"Constellation is {type(constellation)}. Differential is {differential}.")

    def test_tag(self):
        # Send data through bpsk receiver
        # followed by qpsk receiver
        data = [0.9 + 0j, 0.1 + 0.9j, -1 - 0.1j, -0.1 - 0.6j] * 2
        bpsk_data = [1, 1, 0, 0]
        qpsk_data = [1, 3, 0, 0]
        first_tag = gr.tag_t()
        first_tag.key = pmt.intern("set_constellation")
        first_tag.value = digital.bpsk_constellation().as_pmt()
        first_tag.offset = 0
        second_tag = gr.tag_t()
        second_tag.key = pmt.intern("set_constellation")
        second_tag.value = digital.qpsk_constellation().as_pmt()
        second_tag.offset = 4
        src = blocks.vector_source_c(data, False, 1, [first_tag, second_tag])
        decoder = digital.constellation_receiver_cb(
            digital.bpsk_constellation().base(), 0, 0, 0)
        snk = blocks.vector_sink_b()
        tb = gr.top_block()
        tb.connect(src, decoder, snk)
        tb.run()
        self.assertEqual(list(snk.data()), bpsk_data + qpsk_data)


class rec_test_tb(gr.top_block):
    """
    Takes a constellation an runs a generic modulation, channel,
    and generic demodulation.
    """

    def __init__(self, constellation, differential,
                 data_length=None, src_data=None, freq_offset=True):
        """
        Args:
            constellation: a constellation object
            differential: whether differential encoding is used
            data_length: the number of bits of data to use
            src_data: a list of the bits to use
            freq_offset: whether to use a frequency offset in the channel
        """
        super(rec_test_tb, self).__init__()
        # Transmission Blocks
        if src_data is None:
            self.src_data = tuple([random.randint(0, 1)
                                   for i in range(0, data_length)])
        else:
            self.src_data = src_data
        packer = blocks.unpacked_to_packed_bb(1, gr.GR_MSB_FIRST)
        src = blocks.vector_source_b(self.src_data)
        mod = generic_mod(constellation, differential=differential)
        # Channel
        if freq_offset:
            channel = channel_model(
                NOISE_VOLTAGE, FREQUENCY_OFFSET, TIMING_OFFSET)
        else:
            channel = channel_model(NOISE_VOLTAGE, 0, TIMING_OFFSET)
        # Receiver Blocks
        if freq_offset:
            demod = generic_demod(constellation, differential=differential,
                                  freq_bw=FREQ_BW,
                                  phase_bw=PHASE_BW)
        else:
            demod = generic_demod(constellation, differential=differential,
                                  freq_bw=0, phase_bw=0)
        self.dst = blocks.vector_sink_b()
        self.connect(src, packer, mod, channel, demod, self.dst)


if __name__ == '__main__':
    gr_unittest.run(test_constellation_receiver)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest
from gnuradio import blocks, zeromq
from gnuradio import eng_notation

import numpy
import time
import zmq


class qa_zeromq_sub (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()
        self.zmq_context = zmq.Context()
        self.pub_socket = self.zmq_context.socket(zmq.PUB)
        self.pub_socket.bind("tcp://127.0.0.1:0")
        self._address = self.pub_socket.getsockopt(zmq.LAST_ENDPOINT).decode()

    def tearDown(self):
        self.pub_socket.close()
        self.zmq_context.term()
        self.tb = None

    def test_001(self):
        vlen = 10
        src_data = numpy.array(list(range(vlen)) * 100, 'float32')
        zeromq_sub_source = zeromq.sub_source(
            gr.sizeof_float, vlen, self._address)
        sink = blocks.vector_sink_f(vlen)
        self.tb.connect(zeromq_sub_source, sink)

        self.tb.start()
        time.sleep(0.05)
        self.pub_socket.send(src_data.tobytes())
        time.sleep(0.5)
        self.tb.stop()
        self.tb.wait()
        self.assertFloatTuplesAlmostEqual(sink.data(), src_data)

    def test_002(self):
        vlen = 10
        # Construct multipart source data to publish
        raw_data = [
            numpy.array(
                range(vlen),
                'float32') *
            100,
            numpy.array(
                range(
                    vlen,
                    2 *
                    vlen),
                'float32') *
            100]
        src_data = [a.tobytes() for a in raw_data]
        zeromq_sub_source = zeromq.sub_source(
            gr.sizeof_float, vlen, self._address)
        sink = blocks.vector_sink_f(vlen)
        self.tb.connect(zeromq_sub_source, sink)

        self.tb.start()
        time.sleep(0.05)
        self.pub_socket.send_multipart(src_data)
        time.sleep(0.5)
        self.tb.stop()
        self.tb.wait()

        # Source block will concatenate everything together
        expected_data = numpy.concatenate(raw_data)
        self.assertFloatTuplesAlmostEqual(sink.data(), expected_data)

    def test_003(self):
        # Check that message is received when correct key is used
        # Construct multipart source data to publish
        vlen = 10
        raw_data = [
            numpy.array(
                range(vlen),
                'float32') *
            100,
            numpy.array(
                range(
                    vlen,
                    2 *
                    vlen),
                'float32') *
            100]
        src_data = [a.tobytes() for a in raw_data]

        src_data = [b"filter_key"] + src_data

        zeromq_sub_source = zeromq.sub_source(
            gr.sizeof_float, vlen, self._address, key="filter_key")
        sink = blocks.vector_sink_f(vlen)
        self.tb.connect(zeromq_sub_source, sink)

        self.tb.start()
        time.sleep(0.05)
        self.pub_socket.send_multipart(src_data)
        time.sleep(0.5)
        self.tb.stop()
        self.tb.wait()

        # Source block will concatenate everything together
        expected_data = numpy.concatenate(raw_data)
        self.assertFloatTuplesAlmostEqual(sink.data(), expected_data)

    def test_004(self):
        # Test that no message is received when wrong key is used
        vlen = 10
        raw_data = [
            numpy.array(
                range(vlen),
                'float32') *
            100,
            numpy.array(
                range(
                    vlen,
                    2 *
                    vlen),
                'float32') *
            100]
        src_data = [a.tobytes() for a in raw_data]

        src_data = [b"filter_key"] + src_data

        zeromq_sub_source = zeromq.sub_source(
            gr.sizeof_float, vlen, self._address, key="wrong_filter_key")
        sink = blocks.vector_sink_f(vlen)
        self.tb.connect(zeromq_sub_source, sink)

        self.tb.start()
        time.sleep(0.05)
        self.pub_socket.send_multipart(src_data)
        time.sleep(0.5)
        self.tb.stop()
        self.tb.wait()

        assert(len(sink.data()) == 0)


if __name__ == '__main__':
    gr_unittest.run(qa_zeromq_sub)



#!/usr/bin/env python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy

from gnuradio import gr, blocks
# Must use absolute import here because this file is not installed as part
# of the module
from gnuradio.fec import extended_encoder
from gnuradio.fec import extended_decoder


class map_bb(gr.sync_block):
    def __init__(self, bitmap):
        gr.sync_block.__init__(
            self,
            name="map_bb",
            in_sig=[numpy.int8],
            out_sig=[numpy.int8])
        self.bitmap = bitmap

    def work(self, input_items, output_items):
        output_items[0][:] = [self.bitmap[x] for x in input_items[0]]
        return len(output_items[0])


class _qa_helper(gr.top_block):

    def __init__(self, data_size, enc, dec, threading):
        gr.top_block.__init__(self, "_qa_helper")

        self.puncpat = puncpat = '11'

        self.enc = enc
        self.dec = dec
        self.data_size = data_size
        self.threading = threading

        self.ext_encoder = extended_encoder(
            enc, threading=self.threading, puncpat=self.puncpat)
        self.ext_decoder = extended_decoder(dec, threading=self.threading, ann=None,
                                            puncpat=self.puncpat, integration_period=10000)

        self.src = blocks.vector_source_b(
            data_size * [0, 1, 2, 3, 5, 7, 9, 13, 15, 25, 31, 45, 63, 95, 127], False)
        self.unpack = blocks.unpack_k_bits_bb(8)
        self.map = map_bb([-1, 1])
        self.to_float = blocks.char_to_float(1)
        self.snk_input = blocks.vector_sink_b()
        self.snk_output = blocks.vector_sink_b()

        self.connect(self.src, self.unpack, self.ext_encoder)
        self.connect(self.ext_encoder, self.map, self.to_float)
        self.connect(self.to_float, self.ext_decoder)
        self.connect(self.unpack, self.snk_input)
        self.connect(self.ext_decoder, self.snk_output)


if __name__ == '__main__':
    frame_size = 30
    enc = fec.dummy_encoder_make(frame_size * 8)
    #enc = fec.repetition_encoder_make(frame_size*8, 3)
    dec = fec.dummy_decoder.make(frame_size * 8)

    tb = _qa_helper(10 * frame_size, enc, dec, None)
    tb.run()

    errs = 0
    for i, o in zip(tb.snk_input.data(), tb.snk_output.data()):
        if i - o != 0:
            errs += 1

    if errs == 0:
        print("Decoded properly")
    else:
        print("Problem Decoding")



#!/usr/bin/env python
#
# Copyright 2004,2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, fft, filter, blocks

import math


def sin_source_f(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.sin(2. * math.pi * freq * x) for x in t]
    return y


def cos_source_f(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.cos(2. * math.pi * freq * x) for x in t]
    return y


def fir_filter(x, taps, delay):
    y = []
    x2 = (len(taps) - 1) * [0, ] + x
    for i in range(len(x)):
        yi = 0
        for j in range(len(taps)):
            yi += taps[len(taps) - 1 - j] * x2[i + j]
        y.append(complex(x2[i + delay], yi))
    return y


def fir_filter2(x1, x2, taps, delay):
    y = []
    x1_2 = (len(taps) - 1) * [0, ] + x1
    x2_2 = (len(taps) - 1) * [0, ] + x2
    for i in range(len(x2)):
        yi = 0
        for j in range(len(taps)):
            yi += taps[len(taps) - 1 - j] * x2_2[i + j]
        y.append(complex(x1_2[i + delay], yi))
    return y


class test_filter_delay_fc(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_filter_delay_one_input(self):
        tb = self.tb

        sampling_freq = 100

        ntaps = 51
        N = int(ntaps + sampling_freq * 0.10)
        data = sin_source_f(sampling_freq, sampling_freq * 0.10, 1.0, N)
        src1 = blocks.vector_source_f(data)
        dst2 = blocks.vector_sink_c()

        # calculate taps
        taps = filter.firdes.hilbert(ntaps, fft.window.WIN_HAMMING)
        hd = filter.filter_delay_fc(taps)

        expected_result = fir_filter(data, taps, (ntaps - 1) // 2)

        tb.connect(src1, hd)
        tb.connect(hd, dst2)

        tb.run()

        # get output
        result_data = dst2.data()
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)

    def test_002_filter_delay_two_inputs(self):
        # giving the same signal to both the inputs should fetch the same results
        # as above

        tb = self.tb

        sampling_freq = 100
        ntaps = 51
        N = int(ntaps + sampling_freq * 0.10)
        data = sin_source_f(sampling_freq, sampling_freq * 0.10, 1.0, N)
        src1 = blocks.vector_source_f(data)
        dst2 = blocks.vector_sink_c()

        # calculate taps
        taps = filter.firdes.hilbert(ntaps, fft.window.WIN_HAMMING)
        hd = filter.filter_delay_fc(taps)

        expected_result = fir_filter2(data, data, taps, (ntaps - 1) // 2)

        tb.connect(src1, (hd, 0))
        tb.connect(src1, (hd, 1))
        tb.connect(hd, dst2)
        tb.run()

        # get output
        result_data = dst2.data()

        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)

    def test_003_filter_delay_two_inputs(self):
        # give two different inputs
        tb = self.tb

        sampling_freq = 100
        ntaps = 51
        N = int(ntaps + sampling_freq * 0.10)

        data1 = sin_source_f(sampling_freq, sampling_freq * 0.10, 1.0, N)
        data2 = cos_source_f(sampling_freq, sampling_freq * 0.10, 1.0, N)
        src1 = blocks.vector_source_f(data1)
        src2 = blocks.vector_source_f(data2)

        taps = filter.firdes.hilbert(ntaps, fft.window.WIN_HAMMING)
        hd = filter.filter_delay_fc(taps)

        expected_result = fir_filter2(data1, data2, taps, (ntaps - 1) // 2)

        dst2 = blocks.vector_sink_c()

        tb.connect(src1, (hd, 0))
        tb.connect(src2, (hd, 1))
        tb.connect(hd, dst2)

        tb.run()

        # get output
        result_data = dst2.data()

        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)


if __name__ == '__main__':
    gr_unittest.run(test_filter_delay_fc)



from qtpy.QtGui import QPen, QPainter, QBrush, QFont, QFontMetrics
from qtpy.QtCore import Qt, QPointF, QRectF
from qtpy.QtWidgets import QGraphicsItem

from . import colors
from ... import Constants
from ....core.ports import Port as CorePort


class Port(CorePort):
    @classmethod
    def make_cls_with_base(cls, super_cls):
        name = super_cls.__name__
        bases = (super_cls,) + cls.__bases__[:-1]
        namespace = cls.__dict__.copy()
        return type(name, bases, namespace)

    def __init__(self, parent, direction, **n):
        super(self.__class__, self).__init__(parent, direction, **n)
        self.gui = GUIPort(self, direction)

    def remove_clone(self, port):
        self.gui.scene().removeItem(port.gui)
        super(self.__class__, self).remove_clone(port)


class GUIPort(QGraphicsItem):
    """
    The graphical port. Interfaces with its underlying Port object using self.core.
    The GUIPort is always instantiated in the Port constructor.

    Note that this constructor is called before its parent GUIBlock is instantiated,
    which is why we call setParentItem() in create_shapes_and_labels().
    """

    def __init__(self, core, direction, **n):
        self.core = core
        QGraphicsItem.__init__(self)

        self.y_offset = 0
        self.height = 3 * 15.0 if self.core.dtype == 'bus' else 15.0
        self.width = 15.0

        if self.core._dir == "sink":
            self.connection_point = self.mapToScene(QPointF(0.0, self.height / 2.0))
            self.ctrl_point = self.mapToScene(QPointF(0.0, self.height / 2.0) - QPointF(5.0, 0.0))
        else:
            self.connection_point = self.mapToScene(QPointF(self.width, self.height / 2.0))
            self.ctrl_point = self.mapToScene(QPointF(self.width, self.height / 2.0) + QPointF(5.0, 0.0))

        self.setFlag(QGraphicsItem.ItemSendsScenePositionChanges)

        self._border_color = self._bg_color = colors.BLOCK_ENABLED_COLOR

        self.setFlag(QGraphicsItem.ItemStacksBehindParent)
        self.setFlag(QGraphicsItem.ItemIsSelectable)
        self.setAcceptHoverEvents(True)

        self._hovering = False
        self.auto_hide_port_labels = False

        self.font = QFont('Helvetica', 8)
        self.fm = QFontMetrics(self.font)

        # TODO: Move somewhere else? Not necessarily
        self.core.parent_flowgraph.gui.addItem(self)

        """ Dummy blocks instantiate ports only when a connection to them is created. """
        if self.core.parent_block.is_dummy_block:
            self.setParentItem(self.core.parent_block.gui)

    def update_connections(self):
        if self.core._dir == "sink":
            self.connection_point = self.mapToScene(QPointF(-10.0, self.height / 2.0))
            self.ctrl_point = self.mapToScene(QPointF(-10.0, self.height / 2.0) - QPointF(100.0, 0.0))
        else:
            self.connection_point = self.mapToScene(QPointF(self.width, self.height / 2.0))
            self.ctrl_point = self.mapToScene(QPointF(self.width, self.height / 2.0) + QPointF(100.0, 0.0))

        for conn in self.core.connections():
            conn.gui.set_rotation(self.parentItem().rotation())
            conn.gui.update()

    def itemChange(self, change, value):
        self.update_connections()
        return QGraphicsItem.itemChange(self, change, value)

    def create_shapes_and_labels(self):
        self.auto_hide_port_labels = self.core.parent.parent.gui.app.qsettings.value(
            'grc/auto_hide_port_labels', type=bool)
        """
        The GUI port is instantiated before its parent block. Therefore we set the parent here,
        not in the constructor. Exception: dummy block ports, see __init__() above
        """
        if not self.parentItem():
            self.setParentItem(self.core.parent_block.gui)

        self.width = max(15, self.fm.width(self.core.name) * 1.5)
        self._update_colors()
        self.update_connections()

    @property
    def _show_label(self) -> bool:
        return self._hovering or not self.auto_hide_port_labels

    def _update_colors(self):
        """
        Get the color that represents this port's type.
        TODO: Codes differ for ports where the vec length is 1 or greater than 1.
        """
        if not self.core.parent.enabled:
            color = colors.BLOCK_DISABLED_COLOR
        elif self.core.domain == Constants.GR_MESSAGE_DOMAIN:
            color = colors.PORT_TYPE_TO_COLOR.get('message')
        else:
            color = colors.PORT_TYPE_TO_COLOR.get(self.core.dtype) or colors.PORT_TYPE_TO_COLOR.get('')
        self._bg_color = color
        self._border_color = color

    def paint(self, painter, option, widget):
        if self.core.hidden:
            return
        painter.setRenderHint(QPainter.Antialiasing)

        pen = QPen(self._border_color)
        painter.setPen(pen)
        painter.setBrush(QBrush(self._bg_color))

        # TODO: should boundingRect() be used here ?
        if self.core._dir == "sink":
            rect = QRectF(-max(0, self.width - 15), 0, self.width, self.height)
        else:
            rect = QRectF(0, 0, self.width, self.height)
        painter.drawRect(rect)

        # TODO: Adjustments for finer rotation values if required (eg. 15, 30)
        block_rotation = self.parentItem().rotation()

        if self._show_label:
            painter.setPen(QPen(1))
            painter.setFont(self.font)

            # Adjust the painter if parent block is 180 degrees rotated
            if block_rotation == 180:
                painter.translate(self.width, self.height)
                painter.rotate(180)

            if self.core._dir == "sink":
                if block_rotation == 180:
                    painter.drawText(QRectF(max(0, self.width - 15), 0, self.width,
                                     self.height), Qt.AlignCenter, self.core.name)
                else:
                    painter.drawText(QRectF(-max(0, self.width - 15), 0, self.width,
                                     self.height), Qt.AlignCenter, self.core.name)
            else:
                painter.drawText(QRectF(0, 0, self.width, self.height), Qt.AlignCenter, self.core.name)

    def center(self):
        return QPointF(self.x() + self.width / 2, self.y() + self.height / 2)

    def boundingRect(self):
        if self.core._dir == "sink":
            return QRectF(-max(0, self.width - 15), 0, self.width, self.height)
        else:
            return QRectF(0, 0, self.width, self.height)

    def hoverEnterEvent(self, event):
        self._hovering = True
        return QGraphicsItem.hoverEnterEvent(self, event)

    def hoverLeaveEvent(self, event):
        self._hovering = False
        return QGraphicsItem.hoverLeaveEvent(self, event)



#!/usr/bin/env python
#
# Copyright 2008,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks

import math


class test_patterned_interleaver (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        dst_data = [0, 0, 1, 2, 0, 2, 1, 0]
        src0 = blocks.vector_source_f(200 * [0])
        src1 = blocks.vector_source_f(200 * [1])
        src2 = blocks.vector_source_f(200 * [2])
        itg = blocks.patterned_interleaver(gr.sizeof_float, dst_data)
        dst = blocks.vector_sink_f()
        head = blocks.head(gr.sizeof_float, 8)

        self.tb.connect(src0, (itg, 0))
        self.tb.connect(src1, (itg, 1))
        self.tb.connect(src2, (itg, 2))
        self.tb.connect(itg, head, dst)

        self.tb.run()
        self.assertEqual(list(dst_data), list(dst.data()))


if __name__ == '__main__':
    gr_unittest.run(test_patterned_interleaver)



#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio.
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

###############################################################################
# Imports
###############################################################################
from gnuradio import zeromq
#import zeromq
from gnuradio import gr
from gnuradio import blocks
from gnuradio import analog
from gnuradio import eng_notation
from gnuradio.eng_arg import eng_float, intx
from argparse import ArgumentParser
import numpy
import sys
from threading import Thread
import time

###############################################################################
# GNU Radio top_block
###############################################################################


class top_block(gr.top_block):
    def __init__(self, options):
        gr.top_block.__init__(self)

        self.options = options

        # socket addresses
        rpc_adr = "tcp://*:6667"
        probe_adr = "tcp://*:5557"
        source_adr = "tcp://" + self.options.servername + ":5555"

        # blocks
        #self.zmq_source = zeromq.req_source(gr.sizeof_float, 1, source_adr)
        #self.zmq_source = zeromq.pull_source(gr.sizeof_float, 1, source_adr)
        self.zmq_source = zeromq.sub_source(gr.sizeof_float, 1, source_adr)
        #self.zmq_probe = zeromq.push_sink(gr.sizeof_float, 1, probe_adr)
        self.zmq_probe = zeromq.pub_sink(gr.sizeof_float, 1, probe_adr)

        # connects
        self.connect(self.zmq_source, self.zmq_probe)

        # ZeroMQ
        self.rpc_manager = zeromq.rpc_manager()
        self.rpc_manager.set_reply_socket(rpc_adr)
        self.rpc_manager.add_interface("start_fg", self.start_fg)
        self.rpc_manager.add_interface("stop_fg", self.stop_fg)
        self.rpc_manager.start_watcher()

    def start_fg(self):
        print("Start Flowgraph")
        try:
            self.start()
        except RuntimeError:
            print("Can't start, flowgraph already running!")

    def stop_fg(self):
        print("Stop Flowgraph")
        self.stop()
        self.wait()

###############################################################################
# Options Parser
###############################################################################


def parse_args():
    """Argument parser."""
    parser = ArgumentParser()
    parser.add_argument("-s", "--servername", default="localhost",
                        help="Server hostname")
    args = parser.parse_args()
    return args

###############################################################################
# Waiter Thread
###############################################################################


class waiter(Thread):
    """ To keep the program alive when flowgraph is stopped. """

    def run(self):
        while keep_running:
            time.sleep(1)


###############################################################################
# Main
###############################################################################
if __name__ == "__main__":
    args = parse_args()
    tb = top_block(args)
    try:
        # keep the program running when flowgraph is stopped
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        pass
    print("Shutting down flowgraph.")
    tb.rpc_manager.stop_watcher()
    tb.stop()
    tb.wait()
    tb = None



#!/usr/bin/env python
#
# Copyright 2005-2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, filter, blocks
import math
import random
import sys


def random_floats(n):
    r = []
    for x in range(n):
        # r.append(float(random.randint(-32768, 32768)))
        r.append(float(random.random()))
    return tuple(r)


def reference_dec_filter(src_data, decim, taps):
    tb = gr.top_block()
    src = blocks.vector_source_f(src_data)
    op = filter.fir_filter_fff(decim, taps)
    dst = blocks.vector_sink_f()
    tb.connect(src, op, dst)
    tb.run()
    result_data = dst.data()
    tb = None
    return result_data


def reference_interp_filter(src_data, interp, taps):
    tb = gr.top_block()
    src = blocks.vector_source_f(src_data)
    op = filter.interp_fir_filter_fff(interp, taps)
    dst = blocks.vector_sink_f()
    tb.connect(src, op, dst)
    tb.run()
    result_data = dst.data()
    tb = None
    return result_data


def reference_interp_dec_filter(src_data, interp, decim, taps):
    tb = gr.top_block()
    src = blocks.vector_source_f(src_data)
    up = filter.interp_fir_filter_fff(interp, (1,))
    dn = filter.fir_filter_fff(decim, taps)
    dst = blocks.vector_sink_f()
    tb.connect(src, up, dn, dst)
    tb.run()
    result_data = dst.data()
    tb = None
    return result_data


class test_rational_resampler (gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)

    def tearDown(self):
        pass

    def test_000_1_to_1(self):
        taps = (-4, 5)
        src_data = (234, -4, 23, -56, 45, 98, -23, -7)
        xr = (1186, -112, 339, -460, -167, 582)
        expected_result = [float(x) for x in xr]

        tb = gr.top_block()
        src = blocks.vector_source_f(src_data)
        op = filter.rational_resampler_fff(1, 1, taps)
        dst = blocks.vector_sink_f()
        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_001_interp(self):
        taps = [1, 10, 100, 1000, 10000]
        src_data = (0, 2, 3, 5, 7, 11, 13, 17)
        interpolation = 3
        xr = (2, 20, 200, 2003, 20030,
              300, 3005, 30050,
              500, 5007, 50070,
              700, 7011, 70110,
              1100, 11013, 110130,
              1300, 13017, 130170,
              1700.0, 17000.0, 170000.0, 0.0)
        expected_result = [float(x) for x in xr]

        tb = gr.top_block()
        src = blocks.vector_source_f(src_data)
        op = filter.rational_resampler_fff(interpolation, 1, taps)
        dst = blocks.vector_sink_f()
        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_002_interp(self):
        taps = random_floats(31)
        src_data = random_floats(10000)
        interpolation = 3

        expected_result = reference_interp_filter(
            src_data, interpolation, taps)

        tb = gr.top_block()
        src = blocks.vector_source_f(src_data)
        op = filter.rational_resampler_fff(interpolation, 1, taps)
        dst = blocks.vector_sink_f()
        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()
        result_data = dst.data()

        N = 1000
        offset = len(taps) - 1
        self.assertEqual(expected_result[offset:offset + N], result_data[0:N])

    def xtest_003_interp(self):
        taps = random_floats(9)
        src_data = random_floats(10000)
        decimation = 3

        expected_result = reference_dec_filter(src_data, decimation, taps)

        tb = gr.top_block()
        src = blocks.vector_source_f(src_data)
        op = filter.rational_resampler_fff(1, decimation, taps)
        dst = blocks.vector_sink_f()
        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()
        result_data = dst.data()

        N = 10
        offset = 10  # len(taps)-1
        print(expected_result[100 + offset:100 + offset + N])
        print(result_data[100:100 + N])
        #self.assertEqual(expected_result[offset:offset+N], result_data[0:N])

    # FIXME disabled.  Triggers hang on SuSE 10.0
    def xtest_004_decim_random_vals(self):
        MAX_TAPS = 9
        MAX_DECIM = 7
        OUTPUT_LEN = 9

        random.seed(0)    # we want reproducibility

        for ntaps in range(1, MAX_TAPS + 1):
            for decim in range(1, MAX_DECIM + 1):
                for ilen in range(ntaps + decim, ntaps + OUTPUT_LEN * decim):
                    src_data = random_floats(ilen)
                    taps = random_floats(ntaps)
                    expected_result = reference_dec_filter(
                        src_data, decim, taps)

                    tb = gr.top_block()
                    src = blocks.vector_source_f(src_data)
                    op = filter.rational_resampler_fff(1, decim, taps)
                    dst = blocks.vector_sink_f()
                    tb.connect(src, op, dst)
                    tb.run()
                    tb = None
                    result_data = dst.data()

                    L1 = len(result_data)
                    L2 = len(expected_result)
                    L = min(L1, L2)
                    if False:
                        sys.stderr.write(
                            'delta = %2d: ntaps = %d decim = %d ilen = %d\n' %
                            (L2 - L1, ntaps, decim, ilen))
                        sys.stderr.write(
                            '  len(result_data) = %d  len(expected_result) = %d\n' %
                            (len(result_data), len(expected_result)))
                    self.assertEqual(expected_result[0:L], result_data[0:L])

    # FIXME disabled.  Triggers hang on SuSE 10.0

    def xtest_005_interp_random_vals(self):
        MAX_TAPS = 9
        MAX_INTERP = 7
        INPUT_LEN = 9

        random.seed(0)    # we want reproducibility

        for ntaps in range(1, MAX_TAPS + 1):
            for interp in range(1, MAX_INTERP + 1):
                for ilen in range(ntaps, ntaps + INPUT_LEN):
                    src_data = random_floats(ilen)
                    taps = random_floats(ntaps)
                    expected_result = reference_interp_filter(
                        src_data, interp, taps)

                    tb = gr.top_block()
                    src = blocks.vector_source_f(src_data)
                    op = filter.rational_resampler_fff(interp, 1, taps)
                    dst = blocks.vector_sink_f()
                    tb.connect(src, op, dst)
                    tb.run()
                    tb = None
                    result_data = dst.data()
                    L1 = len(result_data)
                    L2 = len(expected_result)
                    L = min(L1, L2)
                    # if True or abs(L1-L2) > 1:
                    if False:
                        sys.stderr.write(
                            'delta = %2d: ntaps = %d interp = %d ilen = %d\n' %
                            (L2 - L1, ntaps, interp, ilen))
                        # sys.stderr.write('  len(result_data) = %d  len(expected_result) = %d\n' %
                        #                 (len(result_data), len(expected_result)))
                    #self.assertEqual(expected_result[0:L], result_data[0:L])
                    # FIXME check first ntaps+1 answers
                    self.assertEqual(
                        expected_result[ntaps + 1:L], result_data[ntaps + 1:L])

    def test_006_interp_decim(self):
        taps = random_floats(31)
        src_data = random_floats(10000)
        interp = 3
        decimation = 2

        expected_result = reference_interp_dec_filter(
            src_data, interp, decimation, taps)

        tb = gr.top_block()
        src = blocks.vector_source_f(src_data)
        op = filter.rational_resampler_fff(interp, decimation, taps)
        dst = blocks.vector_sink_f()
        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()
        result_data = dst.data()

        N = 1000
        offset = len(taps) // 2
        self.assertFloatTuplesAlmostEqual(
            expected_result[offset:offset + N], result_data[0:N], 5)

    def test_007_interp_decim_common_factor(self):
        taps = random_floats(31)
        src_data = random_floats(10000)
        interp = 6
        decimation = 2

        expected_result = reference_interp_dec_filter(
            src_data, interp, decimation, taps)

        tb = gr.top_block()
        src = blocks.vector_source_f(src_data)
        op = filter.rational_resampler_fff(interp, decimation, taps)
        dst = blocks.vector_sink_f()
        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()
        result_data = dst.data()

        N = 1000
        offset = len(taps) // 2
        self.assertFloatTuplesAlmostEqual(
            expected_result[offset:offset + N], result_data[0:N], 5)


if __name__ == '__main__':
    # FIXME: Disabled, see ticket:210
    gr_unittest.run(test_rational_resampler)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, analog, blocks


class test_dpll_bb(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_dpll_bb_001(self):
        # Test set/gets

        period = 1.0
        gain = 0.1
        op = analog.dpll_bb(period, gain)

        op.set_gain(0.2)
        g = op.gain()
        self.assertAlmostEqual(g, 0.2)

        f = op.freq()
        self.assertEqual(1 / period, f)

        d0 = 1.0 - 0.5 * f
        d1 = op.decision_threshold()
        self.assertAlmostEqual(d0, d1)

        p = op.phase()
        self.assertEqual(0, p)

    def test_dpll_bb_002(self):
        period = 4
        gain = 0.1

        src_data = 10 * ((period - 1) * [0, ] + [1, ])
        expected_result = src_data

        src = blocks.vector_source_b(src_data)
        op = analog.dpll_bb(period, gain)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 4)


if __name__ == '__main__':
    gr_unittest.run(test_dpll_bb)



#!/usr/bin/env python
#
# Copyright 2004,2010,2012 Free Software Foundation, Inc.
# Copyright 2025 Marcus Müller <mmueller@gnuradio.org>
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr_unittest, trellis
from gnuradio.gr import major_version, api_version

GR_VERSION = (int(major_version()), int(api_version()))


class test_interleaver(gr_unittest.TestCase):

    def test_001_explicit_interleaver_vector_with_length(self):
        if GR_VERSION >= (3, 11):
            self.skipTest("skipping test of removed constructor")
        # deprecated, since length inherent to vector
        k = 5
        interl = [1, 2, 3, 4, 0]
        de_in = [4, 0, 1, 2, 3]
        i = trellis.interleaver(k, interl)
        self.assertEqual((k, interl, de_in), (i.k(), i.interleaver_indices(), i.deinterleaver_indices()))

    def test_002_explicit_interleaver_vector(self):
        interl = [1, 2, 3, 4, 0]
        de_in = [4, 0, 1, 2, 3]
        i = trellis.interleaver(interl)
        self.assertEqual((interl, de_in), (i.interleaver_indices(), i.deinterleaver_indices()))

    def test_002_random(self):
        k = 1023
        seed = 42
        i = trellis.interleaver(k, seed)
        self.assertSequenceEqual(sorted(i.interleaver_indices()), range(k))
        decorated_indices = [
            (interleave_idx, idx) for idx, interleave_idx in enumerate(i.interleaver_indices())
        ]
        reordered = sorted(decorated_indices)
        deinterleaver = [tup[1] for tup in reordered]
        self.assertSequenceEqual(deinterleaver, i.deinterleaver_indices())


if __name__ == "__main__":
    gr_unittest.run(test_interleaver)



#!/usr/bin/env python
#
# Copyright 2009,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


def makeExpected(history, src_data):
    """ Helper function to make expected result from history and source
    data parameters.
    """
    if (history - 1) > 0:
        expected_data = [0.0] * (history - 1)
        expected_data.extend(src_data[:-(history - 1)])
    else:
        expected_data = src_data
    return [float(x) for x in expected_data]


class test_copy(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_simple_hostbuf_copy(self):
        history = 1
        src_data = 10 * [x for x in range(16384)]
        expected_data = makeExpected(history, src_data)
        src = blocks.vector_source_f(src_data)
        copy = blocks.host_buffer_copy(history)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, copy, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_data, result_data)

    def test_simple_hostbuf_copy_hist100(self):
        history = 100
        src_data = 10 * [x for x in range(16384)]
        expected_data = makeExpected(history, src_data)
        src = blocks.vector_source_f(src_data)
        copy = blocks.host_buffer_copy(history)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, copy, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_data, result_data)

    def test_simple_hostbuf_copy_hist113(self):
        history = 113
        src_data = 10 * [x for x in range(16384)]
        expected_data = makeExpected(history, src_data)
        src = blocks.vector_source_f(src_data)
        copy = blocks.host_buffer_copy(history)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, copy, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_data, result_data)

    def test_simple_hostbuf_copy_hist1057_big(self):
        history = 1057
        src_data = 17 * [x for x in range(91537)]
        expected_data = makeExpected(history, src_data)
        src = blocks.vector_source_f(src_data)
        copy = blocks.host_buffer_copy(history)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, copy, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_data, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_copy)



#!/usr/bin/env python
#
# Copyright 2011-2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import random

from gnuradio import gr, gr_unittest, digital, blocks


class test_binary_slicer_fb(gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_binary_slicer_fb(self):
        expected_result = (0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1)
        src_data = (-1, 1, -1, -1, 1, 1, -1, -1, -1, 1, 1, 1, -1, 1, 1, 1, 1)
        src_data = [s + (1 - random.random())
                    for s in src_data]  # add some noise
        src = blocks.vector_source_f(src_data)
        op = digital.binary_slicer_fb()
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()               # run the graph and wait for it to finish

        actual_result = dst.data()  # fetch the contents of the sink
        # print "actual result", actual_result
        # print "expected result", expected_result
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result)


if __name__ == '__main__':
    gr_unittest.run(test_binary_slicer_fb)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, analog, blocks


class test_fmdet_cf(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_fmdet_cf_001(self):
        # Test set/gets

        fh1 = 10
        fh2 = 20
        fl1 = 1
        fl2 = 2
        scale1 = 3
        scale2 = 4
        op = analog.fmdet_cf(1, fl1, fh1, scale1)

        op.set_freq_range(fl2, fh2)
        lo = op.freq_low()
        hi = op.freq_high()
        f = op.freq()
        self.assertEqual(fl2, lo)
        self.assertEqual(fh2, hi)
        self.assertEqual(0, f)

        op.set_scale(scale2)
        s = op.scale()
        b = op.bias()
        eb = 0.5 * scale2 * (hi + lo) / (hi - lo)
        self.assertEqual(scale2, s)
        self.assertAlmostEqual(eb, b)

    # FIXME: This passes QA, but the it's only based off what the
    # block is saying, not what the values should actually be.
    def est_fmdet_cf_002(self):
        N = 100
        src = analog.sig_source_c(1, analog.GR_SIN_WAVE, 0.2, 1)
        head = blocks.head(gr.sizeof_gr_complex, N)
        op = analog.fmdet_cf(1, 0.1, 0.3, 0.1)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, head, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()[4:N]
        expected_result = (100 - 4) * [-0.21755, ]
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 4)


if __name__ == '__main__':
    gr_unittest.run(test_fmdet_cf)



# Copyright 2017 Free Software Foundation, Inc.
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-2.0-or-later
#


import re
from os import path

try:
    # raise ImportError()
    from lxml import etree
    HAVE_LXML = True
except ImportError:
    import xml.etree.ElementTree as etree
    HAVE_LXML = False


_validator_cache = {None: lambda xml: True}


def _get_validator(dtd=None):
    validator = _validator_cache.get(dtd)
    if not validator:
        if not path.isabs(dtd):
            dtd = path.join(path.dirname(__file__), dtd)
        validator = _validator_cache[dtd] = etree.DTD(dtd).validate
    return validator


def load_lxml(filename, document_type_def=None):
    """Load block description from xml file"""

    try:
        xml_tree = etree.parse(filename)
        _get_validator(document_type_def)
        element = xml_tree.getroot()
    except etree.LxmlError:
        raise ValueError("Failed to parse or validate {}".format(filename))

    version_info = {}
    for inst in xml_tree.xpath('/processing-instruction()'):
        if inst.target == 'grc':
            version_info.update(inst.attrib)

    return element, version_info


def load_stdlib(filename, document_type_def=None):
    """Load block description from xml file"""

    if isinstance(filename, str):
        with open(filename, 'rb') as xml_file:
            data = xml_file.read().decode('utf-8')
    else:  # Already opened
        data = filename.read().decode('utf-8')

    try:
        element = etree.fromstring(data)
    except etree.ParseError:
        raise ValueError("Failed to parse {}".format(filename))

    version_info = {}
    for body in re.findall(r'<\?(.*?)\?>', data):
        inst = etree.fromstring('<' + body + '/>')
        if inst.tag == 'grc':
            version_info.update(inst.attrib)

    return element, version_info


load = load_lxml if HAVE_LXML else load_stdlib



#!/usr/bin/env python

from gnuradio import gr
from gnuradio import audio
from gnuradio import trellis, digital, blocks
from gnuradio import eng_notation
import math
import sys
import random
import fsm_utils

try:
    from gnuradio import analog
except ImportError:
    sys.stderr.write("Error: Program requires gr-analog.\n")
    sys.exit(1)


def run_test(f, Kb, bitspersymbol, K, dimensionality, constellation, N0, seed):
    tb = gr.top_block()

    # TX
    src = blocks.lfsr_32k_source_s()
    src_head = blocks.head(gr.sizeof_short, Kb / 16)  # packet size in shorts
    # unpack shorts to symbols compatible with the FSM input cardinality
    s2fsmi = blocks.packed_to_unpacked_ss(bitspersymbol, gr.GR_MSB_FIRST)
    enc = trellis.encoder_ss(f, 0)  # initial state = 0
    mod = digital.chunks_to_symbols_sf(constellation, dimensionality)

    # CHANNEL
    add = blocks.add_ff()
    noise = analog.noise_source_f(analog.GR_GAUSSIAN, math.sqrt(N0 / 2), seed)

    # RX
    # data preprocessing to generate metrics for Viterbi
    metrics = trellis.metrics_f(
        f.O(), dimensionality, constellation, digital.TRELLIS_EUCLIDEAN)
    # Put -1 if the Initial/Final states are not set.
    va = trellis.viterbi_s(f, K, 0, -1)
    fsmi2s = blocks.unpacked_to_packed_ss(
        bitspersymbol, gr.GR_MSB_FIRST)  # pack FSM input symbols to shorts
    dst = blocks.check_lfsr_32k_s()

    tb.connect(src, src_head, s2fsmi, enc, mod)
    tb.connect(mod, (add, 0))
    tb.connect(noise, (add, 1))
    tb.connect(add, metrics)
    tb.connect(metrics, va, fsmi2s, dst)

    tb.run()

    # A bit of cheating: run the program once and print the
    # final encoder state.
    # Then put it as the last argument in the viterbi block
    # print "final state = " , enc.ST()

    ntotal = dst.ntotal()
    nright = dst.nright()
    runlength = dst.runlength()
    return (ntotal, ntotal - nright)


def main(args):
    nargs = len(args)
    if nargs == 3:
        fname = args[0]
        esn0_db = float(args[1])  # Es/No in dB
        # number of times the experiment is run to collect enough errors
        rep = int(args[2])
    else:
        sys.stderr.write(
            'usage: test_tcm.py fsm_fname Es/No_db  repetitions\n')
        sys.exit(1)

    # system parameters
    f = trellis.fsm(fname)  # get the FSM specification from a file
    # packet size in bits (make it multiple of 16 so it can be packed in a short)
    Kb = 1024 * 16
    # bits per FSM input symbol
    bitspersymbol = int(round(math.log(f.I()) / math.log(2)))
    K = Kb / bitspersymbol  # packet size in trellis steps
    modulation = fsm_utils.psk4  # see fsm_utlis.py for available predefined modulations
    dimensionality = modulation[0]
    constellation = modulation[1]
    if len(constellation) / dimensionality != f.O():
        sys.stderr.write(
            'Incompatible FSM output cardinality and modulation size.\n')
        sys.exit(1)
    # calculate average symbol energy
    Es = 0
    for i in range(len(constellation)):
        Es = Es + constellation[i]**2
    Es = Es / (len(constellation) // dimensionality)
    N0 = Es / pow(10.0, esn0_db / 10.0)  # noise variance

    tot_s = 0
    terr_s = 0
    for i in range(rep):
        # run experiment with different seed to get different noise realizations
        (s, e) = run_test(f, Kb, bitspersymbol, K,
                          dimensionality, constellation, N0, -int(666 + i))
        tot_s = tot_s + s
        terr_s = terr_s + e
        if (i % 100 == 0):
            print(i, s, e, tot_s, terr_s, '%e' % ((1.0 * terr_s) / tot_s))
    # estimate of the (short) error rate
    print(tot_s, terr_s, '%e' % ((1.0 * terr_s) / tot_s))


if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2018-2021 National Technology & Engineering Solutions of Sandia, LLC
# (NTESS). Under the terms of Contract DE-NA0003525 with NTESS, the U.S. Government
# retains certain rights in this software.
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from builtins import range
from gnuradio import gr, gr_unittest, blocks, pdu
import pmt
import time
import numpy


class qa_take_skip_to_pdu_X (gr_unittest.TestCase):

    # this method is necessary because by default pmt.equal does not evaluate
    # the *contents* of a uniform vector
    def assertEqualPDU(self, pdu1, pdu2):
        # first check the equal() function:
        self.assertTrue(pmt.equal(pdu1, pdu2))
        # then check the dictionaries:
        self.assertTrue(pmt.equal(pmt.car(pdu1), pmt.car(pdu2)))
        # then check the elements of the respective vectors
        vec1 = pmt.cdr(pdu1)
        vec2 = pmt.cdr(pdu2)
        self.assertTrue(pmt.equal(vec1, vec2))
        self.assertTrue((pmt.to_python(vec1) == pmt.to_python(vec2)).all(),
                        msg=f"vectors not equal? {vec1!r} {vec2!r}")

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_f_32(self):
        self.source = blocks.vector_source_f(range(0, 32 * 3), False, 1, [])
        self.ts_pdu = pdu.take_skip_to_pdu_f(32, 32)
        self.debug = blocks.message_debug()
        self.tb.connect((self.source, 0), (self.ts_pdu, 0))
        self.tb.msg_connect((self.ts_pdu, 'pdus'), (self.debug, 'store'))

        dic = pmt.dict_add(pmt.make_dict(), pmt.intern(
            "pdu_num"), pmt.from_uint64(0))
        vec = pmt.init_f32vector(32, range(0, 32))
        expected = pmt.cons(dic, vec)
        self.tb.run()
        actual = self.debug.get_message(0)
        self.assertEqualPDU(actual, expected)

    def test_002_c_80(self):
        self.source = blocks.vector_source_c(range(0, 32 * 3), False, 1, [])
        self.ts_pdu = pdu.take_skip_to_pdu_c(80, 32)
        self.debug = blocks.message_debug()
        self.tb.connect((self.source, 0), (self.ts_pdu, 0))
        self.tb.msg_connect((self.ts_pdu, 'pdus'), (self.debug, 'store'))

        dic = pmt.dict_add(pmt.make_dict(), pmt.intern(
            "pdu_num"), pmt.from_uint64(0))
        vec = pmt.init_c32vector(80, range(0, 80))
        expected = pmt.cons(dic, vec)
        self.tb.run()
        actual = self.debug.get_message(0)
        self.assertEqualPDU(actual, expected)

    def test_003_s_2_11_7(self):
        self.source = blocks.vector_source_s(range(0, 32 * 3), False, 1, [])
        self.ts_pdu = pdu.take_skip_to_pdu_s(2, 11)
        self.debug = blocks.message_debug()
        self.tb.connect((self.source, 0), (self.ts_pdu, 0))
        self.tb.msg_connect((self.ts_pdu, 'pdus'), (self.debug, 'store'))

        dic = pmt.dict_add(pmt.make_dict(), pmt.intern(
            "pdu_num"), pmt.from_uint64(7))
        vec = pmt.init_s16vector(2, list(range(91, 93)))
        expected = pmt.cons(dic, vec)
        self.tb.run()
        actual = self.debug.get_message(7)
        self.assertEqualPDU(actual, expected)

    def test_004_b_512(self):
        self.source = blocks.vector_source_b(
            list(range(0, 256)) * 4, False, 1, [])
        self.ts_pdu = pdu.take_skip_to_pdu_b(512, 1)
        self.debug = blocks.message_debug()
        self.tb.connect((self.source, 0), (self.ts_pdu, 0))
        self.tb.msg_connect((self.ts_pdu, 'pdus'), (self.debug, 'store'))

        dic = pmt.dict_add(pmt.make_dict(), pmt.intern(
            "pdu_num"), pmt.from_uint64(0))
        vec = pmt.init_u8vector(512, list(range(0, 256)) * 2)
        expected = pmt.cons(dic, vec)
        self.tb.run()
        actual = self.debug.get_message(0)
        self.assertEqualPDU(actual, expected)


if __name__ == '__main__':
    gr_unittest.run(qa_take_skip_to_pdu_X)



#!/usr/bin/env python
##################################################
# Gnuradio Python Flow Graph
# Title: Second Order Distortion
# Author: mettus
# Generated: Thu Aug  1 12:30:23 2013
##################################################

from gnuradio import blocks
from gnuradio import gr
from gnuradio.filter import firdes
import math


class distortion_2_gen(gr.hier_block2):

    def __init__(self, beta=0):
        gr.hier_block2.__init__(
            self, "Second Order Distortion",
            gr.io_signature(1, 1, gr.sizeof_gr_complex * 1),
            gr.io_signature(1, 1, gr.sizeof_gr_complex * 1),
        )

        ##################################################
        # Parameters
        ##################################################
        self.beta = beta

        ##################################################
        # Blocks
        ##################################################
        self.blocks_multiply_xx_0_0 = blocks.multiply_vcc(1)
        self.blocks_multiply_xx_0 = blocks.multiply_vcc(1)
        self.blocks_multiply_const_vxx_0 = blocks.multiply_const_vcc((beta, ))
        self.blocks_conjugate_cc_0 = blocks.conjugate_cc()
        self.blocks_add_xx_0_0 = blocks.add_vcc(1)
        self.blocks_add_xx_0 = blocks.add_vcc(1)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_multiply_const_vxx_0, 0),
                     (self.blocks_add_xx_0, 1))
        self.connect((self, 0), (self.blocks_multiply_xx_0, 0))
        self.connect((self, 0), (self.blocks_add_xx_0, 0))
        self.connect((self.blocks_add_xx_0, 0), (self, 0))
        self.connect((self, 0), (self.blocks_conjugate_cc_0, 0))
        self.connect((self.blocks_conjugate_cc_0, 0),
                     (self.blocks_multiply_xx_0_0, 1))
        self.connect((self, 0), (self.blocks_multiply_xx_0, 1))
        self.connect((self, 0), (self.blocks_multiply_xx_0_0, 0))
        self.connect((self.blocks_multiply_xx_0_0, 0),
                     (self.blocks_add_xx_0_0, 1))
        self.connect((self.blocks_multiply_xx_0, 0),
                     (self.blocks_add_xx_0_0, 0))
        self.connect((self.blocks_add_xx_0_0, 0),
                     (self.blocks_multiply_const_vxx_0, 0))

    def get_beta(self):
        return self.beta

    def set_beta(self, beta):
        self.beta = beta
        self.blocks_multiply_const_vxx_0.set_k((self.beta, ))



#!/usr/bin/env python
#
# Copyright 2012-2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, fft, filter, blocks

import math


def sig_source_c(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.cos(2. * math.pi * freq * x) +
         1j * math.sin(2. * math.pi * freq * x) for x in t]
    return y


class test_pfb_interpolator(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        N = 1000         # number of samples to use
        M = 5            # Number of channels
        fs = 1000        # baseband sampling rate
        ofs = M * fs       # output samp rate of interpolator

        taps = filter.firdes.low_pass_2(
            M,
            ofs,
            fs / 4,
            fs / 10,
            attenuation_dB=80,
            window=fft.window.WIN_BLACKMAN_hARRIS)

        freq = 123.456
        data = sig_source_c(fs, freq, 1, N)
        signal = blocks.vector_source_c(data)
        pfb = filter.pfb_interpolator_ccf(M, taps)
        snk = blocks.vector_sink_c()

        self.tb.connect(signal, pfb)
        self.tb.connect(pfb, snk)

        self.tb.run()

        Ntest = 50
        L = len(snk.data())

        # Phase rotation through the filters
        phase = 4.8870112969978994

        # Create a time scale
        t = [float(x) / ofs for x in range(0, L)]

        # Create known data as complex sinusoids for the baseband freq
        # of the extracted channel is due to decimator output order.
        expected_data = [
            math.cos(
                2. *
                math.pi *
                freq *
                x +
                phase) +
            1j *
            math.sin(
                2. *
                math.pi *
                freq *
                x +
                phase) for x in t]

        dst_data = snk.data()

        self.assertComplexTuplesAlmostEqual(
            expected_data[-Ntest:], dst_data[-Ntest:], 4)


if __name__ == '__main__':
    gr_unittest.run(test_pfb_interpolator)



# Copyright 2008-2016 Free Software Foundation, Inc.
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-2.0-or-later
#


from .hier_block import HierBlockGenerator, QtHierBlockGenerator
from .top_block import TopBlockGenerator
from .cpp_top_block import CppTopBlockGenerator
from .cpp_hier_block import CppHierBlockGenerator


class Generator(object):
    """Adaptor for various generators (uses generate_options)"""

    def __init__(self, flow_graph, output_dir):
        """
        Initialize the generator object.
        Determine the file to generate.

        Args:
            flow_graph: the flow graph object
            output_dir: the output path for generated files
        """
        self.generate_options = flow_graph.get_option('generate_options')
        self.output_language = flow_graph.get_option('output_language')

        if self.output_language == 'python':

            if self.generate_options == 'hb':
                generator_cls = HierBlockGenerator
            elif self.generate_options == 'hb_qt_gui':
                generator_cls = QtHierBlockGenerator
            else:
                generator_cls = TopBlockGenerator

        elif self.output_language == 'cpp':

            if self.generate_options == 'hb':
                generator_cls = CppHierBlockGenerator
            elif self.generate_options == 'hb_qt_gui':
                pass
            else:
                generator_cls = CppTopBlockGenerator

        self._generator = generator_cls(flow_graph, output_dir)

    def __getattr__(self, item):
        """get all other attrib from actual generator object"""
        return getattr(self._generator, item)



#!/usr/bin/env python
#
# Copyright 2008,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_integrate (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000_ss(self):
        src_data = [1, 2, 3, 4, 5, 6]
        dst_data = [6, 15]
        src = blocks.vector_source_s(src_data)
        itg = blocks.integrate_ss(3)
        dst = blocks.vector_sink_s()
        self.tb.connect(src, itg, dst)
        self.tb.run()
        self.assertEqual(dst_data, dst.data())

    def test_001_ii(self):
        src_data = [1, 2, 3, 4, 5, 6]
        dst_data = [6, 15]
        src = blocks.vector_source_i(src_data)
        itg = blocks.integrate_ii(3)
        dst = blocks.vector_sink_i()
        self.tb.connect(src, itg, dst)
        self.tb.run()
        self.assertEqual(dst_data, dst.data())

    def test_002_ff(self):
        src_data = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]
        dst_data = [6.0, 15.0]
        src = blocks.vector_source_f(src_data)
        itg = blocks.integrate_ff(3)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, itg, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(dst_data, dst.data(), 6)

    def test_003_cc(self):
        src_data = [
            1.0 + 1.0j,
            2.0 + 2.0j,
            3.0 + 3.0j,
            4.0 + 4.0j,
            5.0 + 5.0j,
            6.0 + 6.0j]
        dst_data = [6.0 + 6.0j, 15.0 + 15.0j]
        src = blocks.vector_source_c(src_data)
        itg = blocks.integrate_cc(3)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, itg, dst)
        self.tb.run()
        self.assertComplexTuplesAlmostEqual(dst_data, dst.data(), 6)

    def test_004_ss_vec(self):
        src_data = [1, 2, 3, 4, 5, 6]
        dst_data = [9, 12]
        vlen = 2
        src = blocks.vector_source_s(src_data, False, vlen)
        itg = blocks.integrate_ss(3, vlen)
        dst = blocks.vector_sink_s(vlen)
        self.tb.connect(src, itg, dst)
        self.tb.run()
        self.assertEqual(dst_data, dst.data())

    def test_003_cc_vec(self):
        src_data = [
            1.0 + 1.0j,
            2.0 + 2.0j,
            3.0 + 3.0j,
            4.0 + 4.0j,
            5.0 + 5.0j,
            6.0 + 6.0j]
        dst_data = [9.0 + 9.0j, 12.0 + 12.0j]
        vlen = 2
        src = blocks.vector_source_c(src_data, False, vlen)
        itg = blocks.integrate_cc(3, vlen)
        dst = blocks.vector_sink_c(vlen)
        self.tb.connect(src, itg, dst)
        self.tb.run()
        self.assertComplexTuplesAlmostEqual(dst_data, dst.data(), 6)


if __name__ == '__main__':
    gr_unittest.run(test_integrate)
